from enum import Enum, unique


@unique
class Make(Enum):
    ACADIAN                 = "acadian"
    ACURA                   = "acura"
    ALFA_ROMERO             = "alfa romera"
    AMC                     = "american motors"
    AMERICAN_LAFRANCE       = "american lafrance"
    AUDI                    = "audi"
    AUTOCAR                 = "autocar"
    BENTLEY                 = "bentley"
    BLUEBIRD                = "bluebird"
    BMW                     = "bmw"
    BUICK                   = "buick"
    CADILLAC                = "cadillac"
    CHEVROLET               = "chevrolet"
    CHRYSLER                = "chrysler"
    CROWN                   = "crown"
    DAEWOO                  = "daewoo"
    DATSUN                  = "datsun"
    DELOREAN                = "delorean"
    DODGE                   = "dodge"
    DUCATI                  = "ducati"
    FERRARI                 = "ferrari"
    FIAT                    = "fiat"
    FORD                    = "ford"
    FREIGHTLINER            = "freightliner"
    GEO                     = "geo"
    GILLIG                  = "gillig"
    GMC                     = "gmc"
    GRUMMAN                 = "grumman"
    HARLEY                  = "harley-davidson"
    HINO                    = "hino"
    HONDA                   = "honda"
    HUMMER                  = "hummer"
    HYUNDAI                 = "hyundai"
    INFINITI                = "infiniti"
    INTERNATIONAL_HARVESTER = "international harvester"
    ISUZU                   = "isuzu"
    JAGUAR                  = "jaguar"
    JEEP                    = "jeep"
    JOHN_DEERE              = "john deere"
    KAWASAKI                = "kawasaki"
    KENWORTH                = "kenworth"
    KIA                     = "kia"
    LAND_ROVER              = "land rover"
    LEXUS                   = "lexus"
    LINCOLN                 = "lincoln"
    MACK                    = "mack"
    MASERATI                = "maserati"
    MAZDA                   = "mazda"
    MERCEDES_BENZ           = "mercedes-benz"
    MERCURY                 = "mercury"
    MINI                    = "mini"
    MITSUBISHI              = "mitsubishi"
    NISSAN                  = "nissan"
    NONE                    = None
    OLDSMOBILE              = "oldsmobile"
    PETERBILT               = "peterbilt"
    PLYMOUTH                = "plymouth"
    PONTIAC                 = "pontiac"
    PORSCHE                 = "porsche"
    RADPOWER                = "rad power bikes"
    RAM                     = "ram"
    SAAB                    = "saab"
    SATURN                  = "saturn"
    SCHWINN                 = "schwinn"
    SCION                   = "scion"
    SMART                   = "smart"
    STERLING                = "sterling"
    SUBARU                  = "subaru"
    SUZUKI                  = "suzuki"
    TESLA                   = "tesla"
    THOMAS                  = "thomas"
    TOYOTA                  = "toyota"
    TREK                    = "trek"
    TRIUMPH                 = "triumph"
    VESPA                   = "vespa"
    VOLKSWAGEN              = "volkswagen"
    VOLVO                   = "volvo"
    WHITE                   = "white"
    WINNEBAGO               = "winnebago"
    YAMAHA                  = "yamaha"


MAKE_MAP = {
    "(UNK)": Make.NONE.value,
    "--": Make.NONE.value,
    "---": Make.NONE.value,
    "/": Make.NONE.value,
    "ACAD": Make.ACADIAN.value,
    "ACADIAN": Make.ACADIAN.value,
    "ACCURA": Make.ACURA.value,
    "ACRU": Make.ACURA.value,
    "ACRUA": Make.ACURA.value,
    "ACU": Make.ACURA.value,
    "ACUA": Make.ACURA.value,
    "ACUR /": Make.ACURA.value,
    "ACUR": Make.ACURA.value,
    "ACURA": Make.ACURA.value,
    "ACURA/": Make.ACURA.value,
    "ALFA ROMEO": Make.ALFA_ROMERO.value,
    "ALFA ROMERO": Make.ALFA_ROMERO.value,
    "ALFA": Make.ALFA_ROMERO.value,
    "ALFR": Make.ALFA_ROMERO.value,
    "AMER": Make.AMC.value,
    "AMERI": Make.AMC.value,
    "AMERICAN LA FRANCE": Make.AMERICAN_LAFRANCE.value,
    "AMERICAN MOTORS (AMC)": Make.AMC.value,
    "AMERICAN MOTORS": Make.AMC.value,
    "AMERICAN": Make.AMC.value,
    "AUD": Make.AUDI.value,
    "AUDI /": Make.AUDI.value,
    "AUDI": Make.AUDI.value,
    "AUDI/": Make.AUDI.value,
    "AUDU": Make.AUDI.value,
    "AUID": Make.AUDI.value,
    "AUIDI": Make.AUDI.value,
    "AUTOCAR": Make.AUTOCAR.value,
    "BENT": Make.BENTLEY.value,
    "BENTL": Make.BENTLEY.value,
    "BENTLEY": Make.BENTLEY.value,
    "BENZ": Make.MERCEDES_BENZ.value,
    "BLU BIRD": Make.BLUEBIRD.value,
    "BLU": Make.BLUEBIRD.value,
    "BLUB": Make.BLUEBIRD.value,
    "BLUBD": Make.BLUEBIRD.value,
    "BLUBI": Make.BLUEBIRD.value,
    "BLUBIRD": Make.BLUEBIRD.value,
    "BLUBR": Make.BLUEBIRD.value,
    "BLUBRD": Make.BLUEBIRD.value,
    "BLUE BIR": Make.BLUEBIRD.value,
    "BLUE BIRD": Make.BLUEBIRD.value,
    "BLUE": Make.BLUEBIRD.value,
    "BLUEB": Make.BLUEBIRD.value,
    "BLUEBIR": Make.BLUEBIRD.value,
    "BLUEBIRD (BUS)": Make.BLUEBIRD.value,
    "BLUEBIRD": Make.BLUEBIRD.value,
    "BMW /": Make.BMW.value,
    "BMW": Make.BMW.value,
    "BMW/": Make.BMW.value,
    "BMW1": Make.BMW.value,
    "BMW`": Make.BMW.value,
    "BUCIK": Make.BUICK.value,
    "BUCK": Make.BUICK.value,
    "BUI": Make.BUICK.value,
    "BUIC": Make.BUICK.value,
    "BUICK": Make.BUICK.value,
    "BUICK/": Make.BUICK.value,
    "BUIK": Make.BUICK.value,
    "BWM": Make.BMW.value,
    "BWW": Make.BMW.value,
    "CADI": Make.CADILLAC.value,
    "CADI/": Make.CADILLAC.value,
    "CADIL": Make.CADILLAC.value,
    "CADILA": Make.CADILLAC.value,
    "CADILAC": Make.CADILLAC.value,
    "CADILL": Make.CADILLAC.value,
    "CADILLA": Make.CADILLAC.value,
    "CADILLAC": Make.CADILLAC.value,
    "CEHV": Make.CHEVROLET.value,
    "CEHVY": Make.CHEVROLET.value,
    "CHEROLET": Make.CHEVROLET.value,
    "CHEV /": Make.CHEVROLET.value,
    "CHEV": Make.CHEVROLET.value,
    "CHEV/": Make.CHEVROLET.value,
    "CHEVE": Make.CHEVROLET.value,
    "CHEVER": Make.CHEVROLET.value,
    "CHEVEROL": Make.CHEVROLET.value,
    "CHEVEY": Make.CHEVROLET.value,
    "CHEVOLET": Make.CHEVROLET.value,
    "CHEVR": Make.CHEVROLET.value,
    "CHEVRLET": Make.CHEVROLET.value,
    "CHEVRLT": Make.CHEVROLET.value,
    "CHEVRO": Make.CHEVROLET.value,
    "CHEVROEL": Make.CHEVROLET.value,
    "CHEVROET": Make.CHEVROLET.value,
    "CHEVROL": Make.CHEVROLET.value,
    "CHEVROLE": Make.CHEVROLET.value,
    "CHEVROLET": Make.CHEVROLET.value,
    "CHEVROLT": Make.CHEVROLET.value,
    "CHEVT": Make.CHEVROLET.value,
    "CHEVY": Make.CHEVROLET.value,
    "CHEVY/": Make.CHEVROLET.value,
    "CHEV`": Make.CHEVROLET.value,
    "CHRS": Make.CHRYSLER.value,
    "CHRSLER": Make.CHRYSLER.value,
    "CHRSY": Make.CHRYSLER.value,
    "CHRY": Make.CHEVROLET.value,
    "CHRY/": Make.CHRYSLER.value,
    "CHRYL": Make.CHRYSLER.value,
    "CHRYLER": Make.CHRYSLER.value,
    "CHRYLSER": Make.CHRYSLER.value,
    "CHRYS": Make.CHRYSLER.value,
    "CHRYS/": Make.CHRYSLER.value,
    "CHRYSER": Make.CHRYSLER.value,
    "CHRYSL": Make.CHRYSLER.value,
    "CHRYSLE": Make.CHRYSLER.value,
    "CHRYSLER": Make.CHRYSLER.value,
    "CHRYSLR": Make.CHRYSLER.value,
    "CHRYST": Make.CHRYSLER.value,
    "CHRYSTLE": Make.CHRYSLER.value,
    "CHV": Make.CHEVROLET.value,
    "CHVROLET": Make.CHEVROLET.value,
    "CHVY": Make.CHEVROLET.value,
    "CHY": Make.CHRYSLER.value,
    "CHYRSLER": Make.CHRYSLER.value,
    "CHYSLER": Make.CHRYSLER.value,
    "CROW": Make.CROWN.value,
    "CROWN (BUS)": Make.CROWN.value,
    "CROWN": Make.CROWN.value,
    "CRY": Make.CHRYSLER.value,
    "CRYS": Make.CHRYSLER.value,
    "CRYSLER": Make.CHRYSLER.value,
    "D0DGE": Make.DODGE.value,
    "DAEW": Make.DAEWOO.value,
    "DAEWO": Make.DAEWOO.value,
    "DAEWOO": Make.DAEWOO.value,
    "DATS": Make.DATSUN.value,
    "DATSU": Make.DATSUN.value,
    "DATSUN": Make.DATSUN.value,
    "DATSUN/NISSAN": Make.NISSAN.value,
    "DDGE": Make.DODGE.value,
    "DDOGE": Make.DODGE.value,
    "DELOREAN": Make.DELOREAN.value,
    "DOD": Make.DODGE.value,
    "DODDGE": Make.DODGE.value,
    "DODE": Make.DODGE.value,
    "DODEG": Make.DODGE.value,
    "DODG /": Make.DODGE.value,
    "DODG": Make.DODGE.value,
    "DODG/": Make.DODGE.value,
    "DODGE /": Make.DODGE.value,
    "DODGE": Make.DODGE.value,
    "DODGE/": Make.DODGE.value,
    "DODGER": Make.DODGE.value,
    "DODGE`": Make.DODGE.value,
    "DODGW": Make.DODGE.value,
    "DOG": Make.DODGE.value,
    "DOGDE": Make.DODGE.value,
    "DOGE": Make.DODGE.value,
    "DOGGE": Make.DODGE.value,
    "DUCA": Make.DUCATI.value,
    "DUCAT": Make.DUCATI.value,
    "DUCATI (MOTORCYCLE)": Make.DUCATI.value,
    "DUCATI": Make.DUCATI.value,
    "DUCTI": Make.DUCATI.value,
    "FERRARA": Make.FERRARI.value,
    "FERRARI": Make.FERRARI.value,
    "FIAT": Make.FIAT.value,
    "FIAT-ABARTH": Make.FIAT.value,
    "FOR": Make.FORD.value,
    "FORC": Make.FORD.value,
    "FORD /": Make.FORD.value,
    "FORD": Make.FORD.value,
    "FORD/": Make.FORD.value,
    "FORDE": Make.FORD.value,
    "FORD`": Make.FORD.value,
    "FORE": Make.FORD.value,
    "FORED": Make.FORD.value,
    "FORF": Make.FORD.value,
    "FORN": Make.NONE.value,
    "FORR": Make.FORD.value,
    "FORRD": Make.FORD.value,
    "FORS": Make.FORD.value,
    "FRD": Make.FORD.value,
    "FREHT": Make.FREIGHTLINER.value,
    "FREI /": Make.FREIGHTLINER.value,
    "FREI": Make.FREIGHTLINER.value,
    "FREIG": Make.FREIGHTLINER.value,
    "FREIGH": Make.FREIGHTLINER.value,
    "FREIGHT LINER": Make.FREIGHTLINER.value,
    "FREIGHT": Make.FREIGHTLINER.value,
    "FREIGHTL": Make.FREIGHTLINER.value,
    "FREIGHTLINER CORP": Make.FREIGHTLINER.value,
    "FREIGHTLINER": Make.FREIGHTLINER.value,
    "FREIGT": Make.FREIGHTLINER.value,
    "FREIHT": Make.FREIGHTLINER.value,
    "FREIT": Make.FREIGHTLINER.value,
    "FREITLIN": Make.FREIGHTLINER.value,
    "FREITLNR": Make.FREIGHTLINER.value,
    "FRGH": Make.FREIGHTLINER.value,
    "FRGHT": Make.FREIGHTLINER.value,
    "FRGHT.": Make.FREIGHTLINER.value,
    "FRGHTLNR": Make.FREIGHTLINER.value,
    "FRGT": Make.FREIGHTLINER.value,
    "FRGTH": Make.FREIGHTLINER.value,
    "FRGTLNR": Make.FREIGHTLINER.value,
    "FRH": Make.FREIGHTLINER.value,
    "FRHGT": Make.FREIGHTLINER.value,
    "FRHI": Make.FREIGHTLINER.value,
    "FRHK": Make.FREIGHTLINER.value,
    "FRHT /": Make.FREIGHTLINER.value,
    "FRHT": Make.FREIGHTLINER.value,
    "FRHT.": Make.FREIGHTLINER.value,
    "FRHT/": Make.FREIGHTLINER.value,
    "FRHTL": Make.FREIGHTLINER.value,
    "FRHTLINE": Make.FREIGHTLINER.value,
    "FRHTLINR": Make.FREIGHTLINER.value,
    "FRHTLN": Make.FREIGHTLINER.value,
    "FRHTLNR": Make.FREIGHTLINER.value,
    "FRI": Make.FREIGHTLINER.value,
    "FRIE": Make.FREIGHTLINER.value,
    "FRIEGH": Make.FREIGHTLINER.value,
    "FRIEGHT": Make.FREIGHTLINER.value,
    "FRIEGHTL": Make.FREIGHTLINER.value,
    "FRIET": Make.FREIGHTLINER.value,
    "FRIGH": Make.FREIGHTLINER.value,
    "FRIGHT": Make.FREIGHTLINER.value,
    "FROD": Make.FORD.value,
    "FRT": Make.FREIGHTLINER.value,
    "FRTH": Make.FREIGHTLINER.value,
    "FRTL": Make.FREIGHTLINER.value,
    "FRTLINER": Make.FREIGHTLINER.value,
    "FRTLN": Make.FREIGHTLINER.value,
    "FRTLNR": Make.FREIGHTLINER.value,
    "FTL": Make.FREIGHTLINER.value,
    "FTLR": Make.FREIGHTLINER.value,
    "GENERAL MOTORS CORP": Make.GMC.value,
    "GENERAL": Make.GMC.value,
    "GEO": Make.GEO.value,
    "GILG": Make.GILLIG.value,
    "GILIG": Make.GILLIG.value,
    "GILL": Make.GILLIG.value,
    "GILLI": Make.GILLIG.value,
    "GILLIC": Make.GILLIG.value,
    "GILLIG (BUS)": Make.GILLIG.value,
    "GILLIG BUS": Make.GILLIG.value,
    "GILLIG": Make.GILLIG.value,
    "GM": Make.GMC.value,
    "GMA": Make.GMC.value,
    "GMC (GENERAL MOTORS)": Make.GMC.value,
    "GMC /": Make.GMC.value,
    "GMC": Make.GMC.value,
    "GMC/": Make.GMC.value,
    "GMG": Make.GMC.value,
    "GMS": Make.GMC.value,
    "GMV": Make.GMC.value,
    "GMX": Make.GMC.value,
    "GMZ": Make.GMC.value,
    "GNC": Make.GMC.value,
    "GRUM": Make.GRUMMAN.value,
    "GRUMAN": Make.GRUMMAN.value,
    "GRUMANN": Make.GRUMMAN.value,
    "GRUMIN": Make.GRUMMAN.value,
    "GRUMM": Make.GRUMMAN.value,
    "GRUMMAN MOTOR HOME": Make.GRUMMAN.value,
    "GRUMMAN": Make.GRUMMAN.value,
    "H0NDA": Make.HONDA.value,
    "HANDA": Make.HONDA.value,
    "HARL /": Make.HARLEY.value,
    "HARL DAV": Make.HARLEY.value,
    "HARL": Make.HARLEY.value,
    "HARLE": Make.HARLEY.value,
    "HARLEY D": Make.HARLEY.value,
    "HARLEY DAVIDSON": Make.HARLEY.value,
    "HARLEY": Make.HARLEY.value,
    "HARLEY-D": Make.HARLEY.value,
    "HARLEY-DAVIDSON": Make.HARLEY.value,
    "HARLEYD": Make.HARLEY.value,
    "HARLY": Make.HARLEY.value,
    "HD": Make.HARLEY.value,
    "HD/": Make.HARLEY.value,
    "HINO": Make.HINO.value,
    "HINO/": Make.HINO.value,
    "HIOND": Make.HONDA.value,
    "HIONDA": Make.HONDA.value,
    "HODNA": Make.HONDA.value,
    "HOINDA": Make.HONDA.value,
    "HON": Make.HONDA.value,
    "HONA": Make.HONDA.value,
    "HONAD": Make.HONDA.value,
    "HOND /": Make.HONDA.value,
    "HOND": Make.HONDA.value,
    "HOND/": Make.HONDA.value,
    "HONDA /": Make.HONDA.value,
    "HONDA MC": Make.HONDA.value,
    "HONDA": Make.HONDA.value,
    "HONDA/": Make.HONDA.value,
    "HONDAS": Make.HONDA.value,
    "HONDAY": Make.HONDA.value,
    "HONDA`": Make.HONDA.value,
    "HONDS": Make.HONDA.value,
    "HONE": Make.HONDA.value,
    "HONF": Make.HONDA.value,
    "HONG": Make.HONDA.value,
    "HONS": Make.HONDA.value,
    "HONSA": Make.HONDA.value,
    "HUMM": Make.HUMMER.value,
    "HUMME": Make.HUMMER.value,
    "HUMMER": Make.HUMMER.value,
    "HUMVEE": Make.HUMMER.value,
    "HUN": Make.HYUNDAI.value,
    "HUNDAI": Make.HYUNDAI.value,
    "HUYN": Make.HYUNDAI.value,
    "HUYNDAI": Make.HYUNDAI.value,
    "HYN": Make.HYUNDAI.value,
    "HYND": Make.HYUNDAI.value,
    "HYNDAI": Make.HYUNDAI.value,
    "HYNU": Make.HYUNDAI.value,
    "HYNUDAI": Make.HYUNDAI.value,
    "HYU N": Make.HYUNDAI.value,
    "HYU": Make.HYUNDAI.value,
    "HYUAN": Make.HYUNDAI.value,
    "HYUANDAI": Make.HYUNDAI.value,
    "HYUD": Make.HYUNDAI.value,
    "HYUDAI": Make.HYUNDAI.value,
    "HYUIN": Make.HYUNDAI.value,
    "HYUM": Make.HYUNDAI.value,
    "HYUN /": Make.HYUNDAI.value,
    "HYUN": Make.HYUNDAI.value,
    "HYUN/": Make.HYUNDAI.value,
    "HYUNA": Make.HYUNDAI.value,
    "HYUNAI": Make.HYUNDAI.value,
    "HYUND": Make.HYUNDAI.value,
    "HYUNDA": Make.HYUNDAI.value,
    "HYUNDAI": Make.HYUNDAI.value,
    "HYUNDAI/": Make.HYUNDAI.value,
    "HYUNDAU": Make.HYUNDAI.value,
    "HYUNDAY": Make.HYUNDAI.value,
    "HYUNDI": Make.HYUNDAI.value,
    "HYUNDIA": Make.HYUNDAI.value,
    "HYUU": Make.HYUNDAI.value,
    "HYUUN": Make.HYUNDAI.value,
    "INF": Make.INFINITI.value,
    "INFI /": Make.INFINITI.value,
    "INFI": Make.INFINITI.value,
    "INFIN": Make.INFINITI.value,
    "INFIN/": Make.INFINITI.value,
    "INFINI": Make.INFINITI.value,
    "INFINIT": Make.INFINITI.value,
    "INFINITE": Make.INFINITI.value,
    "INFINITI": Make.INFINITI.value,
    "INFINITY": Make.INFINITI.value,
    "INFINT": Make.INFINITI.value,
    "INFINTI": Make.INFINITI.value,
    "INFINTY": Make.INFINITI.value,
    "INFIT": Make.INFINITI.value,
    "INIF": Make.INFINITI.value,
    "INIFI": Make.INFINITI.value,
    "INIFINIT": Make.INFINITI.value,
    "INIFNITI": Make.INFINITI.value,
    "INTER": Make.INTERNATIONAL_HARVESTER.value,
    "INTERNAT": Make.INTERNATIONAL_HARVESTER.value,
    "INTERNATIONAL HARVESTER": Make.INTERNATIONAL_HARVESTER.value,
    "INTL": Make.NONE.value,
    "ISU": Make.ISUZU.value,
    "ISUZ": Make.ISUZU.value,
    "ISUZU": Make.ISUZU.value,
    "JAG": Make.JAGUAR.value,
    "JAGA": Make.JAGUAR.value,
    "JAGU": Make.JAGUAR.value,
    "JAGUA": Make.JAGUAR.value,
    "JAGUAR": Make.JAGUAR.value,
    "JDEER": Make.JOHN_DEERE.value,
    "JEE": Make.JEEP.value,
    "JEEEP": Make.JEEP.value,
    "JEEF": Make.JEEP.value,
    "JEEO": Make.JEEP.value,
    "JEEP /": Make.JEEP.value,
    "JEEP": Make.JEEP.value,
    "JEEP/": Make.JEEP.value,
    "JEPP": Make.JEEP.value,
    "JOHN DEE": Make.JOHN_DEERE.value,
    "JOHN DEER": Make.JOHN_DEERE.value,
    "JOHN DEERE": Make.JOHN_DEERE.value,
    "JOHN": Make.JOHN_DEERE.value,
    "JOHND": Make.JOHN_DEERE.value,
    "JOHNDEER": Make.JOHN_DEERE.value,
    "KAWA": Make.KAWASAKI.value,
    "KAWAI": Make.KAWASAKI.value,
    "KAWAK": Make.KAWASAKI.value,
    "KAWAS": Make.KAWASAKI.value,
    "KAWASA": Make.KAWASAKI.value,
    "KAWASAK": Make.KAWASAKI.value,
    "KAWASAKI": Make.KAWASAKI.value,
    "KAWASKI": Make.KAWASAKI.value,
    "KAWI": Make.KAWASAKI.value,
    "KAWK": Make.KAWASAKI.value,
    "KENW": Make.KENWORTH.value,
    "KENWO": Make.KENWORTH.value,
    "KENWOR": Make.KENWORTH.value,
    "KENWORT": Make.KENWORTH.value,
    "KENWORTH": Make.KENWORTH.value,
    "KENWRTH": Make.KENWORTH.value,
    "KIA /": Make.KIA.value,
    "KIA": Make.KIA.value,
    "KIA/": Make.KIA.value,
    "KIO": Make.KIA.value,
    "KIS": Make.KIA.value,
    "LAND ROVER": Make.LAND_ROVER.value,
    "LAND RVR": Make.LAND_ROVER.value,
    "LAND": Make.LAND_ROVER.value,
    "LANDR": Make.LAND_ROVER.value,
    "LANDRO": Make.LAND_ROVER.value,
    "LANDROVE": Make.LAND_ROVER.value,
    "LANDROVER": Make.LAND_ROVER.value,
    "LANDRVR": Make.LAND_ROVER.value,
    "LES": Make.LEXUS.value,
    "LESU": Make.LEXUS.value,
    "LEX": Make.LEXUS.value,
    "LEXAS": Make.LEXUS.value,
    "LEXI": Make.LEXUS.value,
    "LEXIS": Make.LEXUS.value,
    "LEXS /": Make.LEXUS.value,
    "LEXS": Make.LEXUS.value,
    "LEXSS": Make.LEXUS.value,
    "LEXSUS": Make.LEXUS.value,
    "LEXU /": Make.LEXUS.value,
    "LEXU": Make.LEXUS.value,
    "LEXUS": Make.LEXUS.value,
    "LEXUS/": Make.LEXUS.value,
    "LEXUX": Make.LEXUS.value,
    "LEZ": Make.LEXUS.value,
    "LEZUS": Make.LEXUS.value,
    "LICOLN": Make.LINCOLN.value,
    "LIN": Make.LINCOLN.value,
    "LINC /": Make.LINCOLN.value,
    "LINC": Make.LINCOLN.value,
    "LINCL": Make.LINCOLN.value,
    "LINCO": Make.LINCOLN.value,
    "LINCOL": Make.LINCOLN.value,
    "LINCOLN CONTINENTAL": Make.LINCOLN.value,
    "LINCOLN": Make.LINCOLN.value,
    "LINCOLN/": Make.LINCOLN.value,
    "LINCON": Make.LINCOLN.value,
    "LND RVR": Make.LAND_ROVER.value,
    "LNDR": Make.LAND_ROVER.value,
    "LNDRVR": Make.LAND_ROVER.value,
    "LUXUS": Make.LEXUS.value,
    "LXS": Make.LEXUS.value,
    "MACK": Make.MACK.value,
    "MADA": Make.MAZDA.value,
    "MADZA": Make.MAZDA.value,
    "MASE": Make.MASERATI.value,
    "MASER": Make.MASERATI.value,
    "MASERATI": Make.MASERATI.value,
    "MASERATT": Make.MASERATI.value,
    "MASI": Make.MASERATI.value,
    "MAXDA": Make.MAZDA.value,
    "MAZ": Make.MAZDA.value,
    "MAZA": Make.MAZDA.value,
    "MAZAD": Make.MAZDA.value,
    "MAZADA": Make.MAZDA.value,
    "MAZD /": Make.MAZDA.value,
    "MAZD": Make.MAZDA.value,
    "MAZDA /": Make.MAZDA.value,
    "MAZDA 3": Make.MAZDA.value,
    "MAZDA 6": Make.MAZDA.value,
    "MAZDA": Make.MAZDA.value,
    "MAZDA/": Make.MAZDA.value,
    "MAZDZ": Make.MAZDA.value,
    "MAZERATI": Make.MASERATI.value,
    "MERB /": Make.MERCEDES_BENZ.value,
    "MERB": Make.MERCEDES_BENZ.value,
    "MERB.": Make.MERCEDES_BENZ.value,
    "MERB/": Make.MERCEDES_BENZ.value,
    "MERBENZ": Make.MERCEDES_BENZ.value,
    "MERBNZ": Make.MERCEDES_BENZ.value,
    "MERC": Make.MERCURY.value,
    "MERCE": Make.MERCEDES_BENZ.value,
    "MERCED": Make.MERCEDES_BENZ.value,
    "MERCEDE": Make.MERCEDES_BENZ.value,
    "MERCEDES BENZ": Make.MERCEDES_BENZ.value,
    "MERCEDES": Make.MERCEDES_BENZ.value,
    "MERCEDES-BENZ": Make.MERCEDES_BENZ.value,
    "MERCEDEZ": Make.MERCEDES_BENZ.value,
    "MERCEDS": Make.MERCEDES_BENZ.value,
    "MERCU": Make.MERCURY.value,
    "MERCUR": Make.MERCURY.value,
    "MERCURY": Make.MERCURY.value,
    "MERD": Make.MERCURY.value,
    "MERZ /": Make.MERCEDES_BENZ.value,
    "MERZ BNZ": Make.MERCEDES_BENZ.value,
    "MERZ": Make.MERCEDES_BENZ.value,
    "MERZ/": Make.MERCEDES_BENZ.value,
    "MERZB": Make.MERCEDES_BENZ.value,
    "MINI COOPER": Make.MINI.value,
    "MINI": Make.MINI.value,
    "MINN": Make.MINI.value,
    "MINNI": Make.MINI.value,
    "MISCELLANEOUS": Make.NONE.value,
    "MISSAN": Make.NISSAN.value,
    "MIST": Make.MITSUBISHI.value,
    "MISTU": Make.MITSUBISHI.value,
    "MIT": Make.MITSUBISHI.value,
    "MITI": Make.MITSUBISHI.value,
    "MITS /": Make.MITSUBISHI.value,
    "MITS": Make.MITSUBISHI.value,
    "MITS.": Make.MITSUBISHI.value,
    "MITS/": Make.MITSUBISHI.value,
    "MITSH": Make.MITSUBISHI.value,
    "MITSU": Make.MITSUBISHI.value,
    "MITSUB": Make.MITSUBISHI.value,
    "MITSUBI": Make.MITSUBISHI.value,
    "MITSUBIS": Make.MITSUBISHI.value,
    "MITSUBISHI": Make.MITSUBISHI.value,
    "MITT": Make.MITSUBISHI.value,
    "MITTS": Make.MITSUBISHI.value,
    "MITU": Make.MITSUBISHI.value,
    "MITZ": Make.MITSUBISHI.value,
    "MNI": Make.MINI.value,
    "MNICP": Make.MINI.value,
    "MNNI": Make.MINI.value,
    "MZD": Make.MAZDA.value,
    "MZDA": Make.MAZDA.value,
    "N/A": Make.NONE.value,
    "NII": Make.NISSAN.value,
    "NIIS": Make.NISSAN.value,
    "NIISAN": Make.NISSAN.value,
    "NIISS": Make.NISSAN.value,
    "NIISSAN": Make.NISSAN.value,
    "NIS": Make.NISSAN.value,
    "NISA": Make.NISSAN.value,
    "NISAA": Make.NISSAN.value,
    "NISAAN": Make.NISSAN.value,
    "NISAN": Make.NISSAN.value,
    "NISAS": Make.NISSAN.value,
    "NISS /": Make.NISSAN.value,
    "NISS": Make.NISSAN.value,
    "NISS/": Make.NISSAN.value,
    "NISSA N": Make.NISSAN.value,
    "NISSA": Make.NISSAN.value,
    "NISSAM": Make.NISSAN.value,
    "NISSAN /": Make.NISSAN.value,
    "NISSAN": Make.NISSAN.value,
    "NISSAN/": Make.NISSAN.value,
    "NISSANA": Make.NISSAN.value,
    "NISSAN`": Make.NISSAN.value,
    "NISSAS": Make.NISSAN.value,
    "NISSASN": Make.NISSAN.value,
    "NISSI": Make.NISSAN.value,
    "NISSIAN": Make.NISSAN.value,
    "NISSN": Make.NISSAN.value,
    "NISSNA": Make.NISSAN.value,
    "NISSS": Make.NISSAN.value,
    "NISSSAN": Make.NISSAN.value,
    "NOT STATED": Make.NONE.value,
    "ODYSSEY": Make.HONDA.value,
    "OLDS": Make.OLDSMOBILE.value,
    "OLDSM": Make.OLDSMOBILE.value,
    "OLDSMO": Make.OLDSMOBILE.value,
    "OLDSMOBI": Make.OLDSMOBILE.value,
    "OLDSMOBILE": Make.OLDSMOBILE.value,
    "OLS": Make.OLDSMOBILE.value,
    "OTHER - ATV": Make.NONE.value,
    "OTHER - AUTO": Make.NONE.value,
    "OTHER - BUS": Make.NONE.value,
    "OTHER - DOMESTIC": Make.NONE.value,
    "OTHER - MOPED": Make.NONE.value,
    "OTHER - MOTORCYCLE": Make.NONE.value,
    "OTHER - MOTORHOME": Make.NONE.value,
    "OTHER - PICKUP": Make.NONE.value,
    "OTHER - SCHOOL BUS": Make.NONE.value,
    "OTHER - TRUCK": Make.NONE.value,
    "OTHER DOMESTICS": Make.NONE.value,
    "OTHER FOREIGN": Make.NONE.value,
    "OTHER": Make.NONE.value,
    "PETE": Make.PETERBILT.value,
    "PETEBILT": Make.PETERBILT.value,
    "PETER": Make.PETERBILT.value,
    "PETERB": Make.PETERBILT.value,
    "PETERBI": Make.PETERBILT.value,
    "PETERBIL": Make.PETERBILT.value,
    "PETERBILT": Make.PETERBILT.value,
    "PETERBL": Make.PETERBILT.value,
    "PETERBLT": Make.PETERBILT.value,
    "PETERBU": Make.PETERBILT.value,
    "PETERBUI": Make.PETERBILT.value,
    "PETERBUILT": Make.PETERBILT.value,
    "PETERBUL": Make.PETERBILT.value,
    "PETKT": Make.PETERBILT.value,
    "PETR": Make.PETERBILT.value,
    "PETRB": Make.PETERBILT.value,
    "PETRBILT": Make.PETERBILT.value,
    "PETRBLT": Make.PETERBILT.value,
    "PLY": Make.PLYMOUTH.value,
    "PLYM": Make.PLYMOUTH.value,
    "PLYMO": Make.PLYMOUTH.value,
    "PLYMOTH": Make.PLYMOUTH.value,
    "PLYMOU": Make.PLYMOUTH.value,
    "PLYMOUTH": Make.PLYMOUTH.value,
    "PONI": Make.PONTIAC.value,
    "PONIT": Make.PONTIAC.value,
    "PONITAC": Make.PONTIAC.value,
    "PONT": Make.PONTIAC.value,
    "PONTAIC": Make.PONTIAC.value,
    "PONTI": Make.PONTIAC.value,
    "PONTIA": Make.PONTIAC.value,
    "PONTIAC": Make.PONTIAC.value,
    "PONTIAC/": Make.PONTIAC.value,
    "PONTIC": Make.PONTIAC.value,
    "POR": Make.PORSCHE.value,
    "PORC": Make.PORSCHE.value,
    "PORCH": Make.PORSCHE.value,
    "PORCHE": Make.PORSCHE.value,
    "PORS": Make.PORSCHE.value,
    "PORSC": Make.PORSCHE.value,
    "PORSCE": Make.PORSCHE.value,
    "PORSCH": Make.PORSCHE.value,
    "PORSCHE": Make.PORSCHE.value,
    "PORSCHE/": Make.PORSCHE.value,
    "PORSE": Make.PORSCHE.value,
    "PORSH": Make.PORSCHE.value,
    "PORSHE": Make.PORSCHE.value,
    "PRIUS": Make.TOYOTA.value,
    "PRTB": Make.PETERBILT.value,
    "PTB": Make.PETERBILT.value,
    "PTBL": Make.PETERBILT.value,
    "PTBLT": Make.PETERBILT.value,
    "PTBR": Make.PETERBILT.value,
    "PTBT": Make.PETERBILT.value,
    "PTE": Make.PETERBILT.value,
    "PTER": Make.PETERBILT.value,
    "PTR": Make.PETERBILT.value,
    "PTRB /": Make.PETERBILT.value,
    "PTRB": Make.PETERBILT.value,
    "PTRB/": Make.PETERBILT.value,
    "PTRBILT": Make.PETERBILT.value,
    "PTRBL": Make.PETERBILT.value,
    "PTRBLT": Make.PETERBILT.value,
    "PTRBT": Make.PETERBILT.value,
    "PTRBUILT": Make.PETERBILT.value,
    "RAD CITY": Make.RADPOWER.value,
    "RAD POWE": Make.RADPOWER.value,
    "RAD": Make.RADPOWER.value,
    "RADPOWER": Make.RADPOWER.value,
    "RADROVER": Make.RADPOWER.value,
    "RAM 2500": Make.RAM.value,
    "RAM": Make.RAM.value,
    "RAM/": Make.RAM.value,
    "RAN": Make.RAM.value,
    "RANG": Make.LAND_ROVER.value,
    "RANGE RO": Make.LAND_ROVER.value,
    "RANGE ROVER": Make.LAND_ROVER.value,
    "RANGE RV": Make.LAND_ROVER.value,
    "RANGE": Make.LAND_ROVER.value,
    "RANGER": Make.LAND_ROVER.value,
    "RANGEROV": Make.LAND_ROVER.value,
    "RNG ROVR": Make.LAND_ROVER.value,
    "RNG RVR": Make.LAND_ROVER.value,
    "RNGRV": Make.LAND_ROVER.value,
    "RNGRVR": Make.LAND_ROVER.value,
    "RORD": Make.FORD.value,
    "ROVER": Make.LAND_ROVER.value,
    "SAAB": Make.SAAB.value,
    "SATN": Make.SATURN.value,
    "SATR": Make.SATURN.value,
    "SATRN": Make.SATURN.value,
    "SATRU": Make.SATURN.value,
    "SATRUN": Make.SATURN.value,
    "SATU": Make.SATURN.value,
    "SATUN": Make.SATURN.value,
    "SATUR": Make.SATURN.value,
    "SATURN": Make.SATURN.value,
    "SATURN/": Make.SATURN.value,
    "SATY": Make.SATURN.value,
    "SCHW": Make.SCHWINN.value,
    "SCHWIN": Make.SCHWINN.value,
    "SCHWINN": Make.SCHWINN.value,
    "SCHWYNN": Make.SCHWINN.value,
    "SCIO": Make.SCION.value,
    "SCIOIN": Make.SCION.value,
    "SCION": Make.SCION.value,
    "SCOIN": Make.SCION.value,
    "SCWHINN": Make.SCHWINN.value,
    "SCWINN": Make.SCHWINN.value,
    "SHWIN": Make.SCHWINN.value,
    "SHWINN": Make.SCHWINN.value,
    "SMAR": Make.SMART.value,
    "SMART": Make.SMART.value,
    "STERLI": Make.STERLING.value,
    "STERLIN": Make.STERLING.value,
    "STERLING": Make.STERLING.value,
    "STRN /": Make.SATURN.value,
    "STRN": Make.SATURN.value,
    "STURN": Make.SATURN.value,
    "SUB": Make.SUBARU.value,
    "SUBA /": Make.SUBARU.value,
    "SUBA": Make.SUBARU.value,
    "SUBAR": Make.SUBARU.value,
    "SUBARA": Make.SUBARU.value,
    "SUBARAU": Make.SUBARU.value,
    "SUBARI": Make.SUBARU.value,
    "SUBARU": Make.SUBARU.value,
    "SUBARU/": Make.SUBARU.value,
    "SUBARY": Make.SUBARU.value,
    "SUBI": Make.SUBARU.value,
    "SUBN": Make.SUBARU.value,
    "SUBR": Make.SUBARU.value,
    "SUBRA": Make.SUBARU.value,
    "SUBRARU": Make.SUBARU.value,
    "SUBRAU": Make.SUBARU.value,
    "SUBRU": Make.SUBARU.value,
    "SUBU": Make.SUBARU.value,
    "SUBUARU": Make.SUBARU.value,
    "SUBUR": Make.SUBARU.value,
    "SUBURA": Make.SUBARU.value,
    "SUBURU": Make.SUBARU.value,
    "SUS": Make.SUZUKI.value,
    "SUSUKI": Make.SUZUKI.value,
    "SUV": Make.NONE.value,
    "SUZ": Make.SUZUKI.value,
    "SUZI": Make.SUZUKI.value,
    "SUZIKI": Make.SUZUKI.value,
    "SUZK": Make.SUZUKI.value,
    "SUZKI": Make.SUZUKI.value,
    "SUZU /": Make.SUZUKI.value,
    "SUZU": Make.SUZUKI.value,
    "SUZUK": Make.SUZUKI.value,
    "SUZUKI MC": Make.SUZUKI.value,
    "SUZUKI": Make.SUZUKI.value,
    "SUZUKI/": Make.SUZUKI.value,
    "T0Y": Make.TOYOTA.value,
    "T0YOTA": Make.TOYOTA.value,
    "TAHOE": Make.GMC.value,
    "TAOTA": Make.TOYOTA.value,
    "TAOTAO": Make.TOYOTA.value,
    "TESL": Make.TESLA.value,
    "TESLA MOTORS": Make.TESLA.value,
    "TESLA": Make.TESLA.value,
    "TESLA/": Make.TESLA.value,
    "THOM": Make.THOMAS.value,
    "THOMA": Make.THOMAS.value,
    "THOMAS (BUS)": Make.THOMAS.value,
    "THOMAS B": Make.THOMAS.value,
    "THOMAS": Make.THOMAS.value,
    "TOT": Make.TOYOTA.value,
    "TOTA": Make.TOYOTA.value,
    "TOTO": Make.TOYOTA.value,
    "TOTOTA": Make.TOYOTA.value,
    "TOTOYA": Make.TOYOTA.value,
    "TOTOYTA": Make.TOYOTA.value,
    "TOTY": Make.TOYOTA.value,
    "TOTYOA": Make.TOYOTA.value,
    "TOTYOTA": Make.TOYOTA.value,
    "TOY": Make.TOYOTA.value,
    "TOY/SCIO": Make.TOYOTA.value,
    "TOY0": Make.TOYOTA.value,
    "TOY0TA": Make.TOYOTA.value,
    "TOYA": Make.TOYOTA.value,
    "TOYAT": Make.TOYOTA.value,
    "TOYATA": Make.TOYOTA.value,
    "TOYI": Make.TOYOTA.value,
    "TOYO /": Make.TOYOTA.value,
    "TOYO": Make.TOYOTA.value,
    "TOYO/": Make.TOYOTA.value,
    "TOYO/SCI": Make.TOYOTA.value,
    "TOYOA": Make.TOYOTA.value,
    "TOYOAT": Make.TOYOTA.value,
    "TOYORA": Make.TOYOTA.value,
    "TOYOT": Make.TOYOTA.value,
    "TOYOTA": Make.TOYOTA.value,
    "TOYOTA/": Make.TOYOTA.value,
    "TOYOTAS": Make.TOYOTA.value,
    "TOYOTA`": Make.TOYOTA.value,
    "TOYOTO": Make.TOYOTA.value,
    "TOYOTOA": Make.TOYOTA.value,
    "TOYOTR": Make.TOYOTA.value,
    "TOYOTRA": Make.TOYOTA.value,
    "TOYOTS": Make.TOYOTA.value,
    "TOYOTYA": Make.TOYOTA.value,
    "TOYOY": Make.TOYOTA.value,
    "TOYOYA": Make.TOYOTA.value,
    "TOYOYTA": Make.TOYOTA.value,
    "TOYO`": Make.TOYOTA.value,
    "TOYR": Make.TOYOTA.value,
    "TOYT /": Make.TOYOTA.value,
    "TOYT": Make.TOYOTA.value,
    "TOYT.": Make.TOYOTA.value,
    "TOYT/": Make.TOYOTA.value,
    "TOYT/SCI": Make.TOYOTA.value,
    "TOYTA": Make.TOYOTA.value,
    "TOYTO": Make.TOYOTA.value,
    "TOYTOA": Make.TOYOTA.value,
    "TOYTOTA": Make.TOYOTA.value,
    "TOYTT": Make.TOYOTA.value,
    "TOYY": Make.TOYOTA.value,
    "TREC": Make.TREK.value,
    "TRECK": Make.TREK.value,
    "TREK": Make.TREK.value,
    "TREK.value, INC.": Make.TREK.value,
    "TRIPH": Make.TRIUMPH.value,
    "TRIU": Make.TRIUMPH.value,
    "TRIUM": Make.TRIUMPH.value,
    "TRIUMP": Make.TRIUMPH.value,
    "TRIUMPH": Make.TRIUMPH.value,
    "TRIUPH": Make.TRIUMPH.value,
    "TRUIMPH": Make.TRIUMPH.value,
    "TRUM": Make.TRIUMPH.value,
    "TSLA": Make.TESLA.value,
    "TSMR": Make.TESLA.value,
    "TUNDRA": Make.TOYOTA.value,
    "TYOT": Make.TOYOTA.value,
    "TYOTA": Make.TOYOTA.value,
    "UBER": Make.NONE.value,
    "UKN": Make.NONE.value,
    "UKNONWN": Make.NONE.value,
    "UKNOWN": Make.NONE.value,
    "UNK /": Make.NONE.value,
    "UNK": Make.NONE.value,
    "UNK.": Make.NONE.value,
    "UNK/": Make.NONE.value,
    "UNKN": Make.NONE.value,
    "UNKN/": Make.NONE.value,
    "UNKNONW": Make.NONE.value,
    "UNKNOW": Make.NONE.value,
    "UNKNOWN": Make.NONE.value,
    "UNKNOWN/": Make.NONE.value,
    "UNKNWN": Make.NONE.value,
    "UNKNWON": Make.NONE.value,
    "UNKOWN": Make.NONE.value,
    "UNKWN": Make.NONE.value,
    "UNNKNOWN": Make.NONE.value,
    "UNNOWN": Make.NONE.value,
    "V & W": Make.VOLKSWAGEN.value,
    "V W": Make.VOLKSWAGEN.value,
    "V.W.": Make.VOLKSWAGEN.value,
    "V/W": Make.VOLKSWAGEN.value,
    "VESP": Make.VESPA.value,
    "VESPA": Make.VESPA.value,
    "VOK": Make.VOLKSWAGEN.value,
    "VOKS": Make.VOLKSWAGEN.value,
    "VOLCO": Make.VOLVO.value,
    "VOLK /": Make.VOLKSWAGEN.value,
    "VOLK": Make.VOLKSWAGEN.value,
    "VOLK/": Make.VOLKSWAGEN.value,
    "VOLKD": Make.VOLKSWAGEN.value,
    "VOLKL": Make.VOLKSWAGEN.value,
    "VOLKS": Make.VOLKSWAGEN.value,
    "VOLKS/": Make.VOLKSWAGEN.value,
    "VOLKSW": Make.VOLKSWAGEN.value,
    "VOLKSWA": Make.VOLKSWAGEN.value,
    "VOLKSWAG": Make.VOLKSWAGEN.value,
    "VOLKSWAGEN": Make.VOLKSWAGEN.value,
    "VOLKSWAGON": Make.VOLKSWAGEN.value,
    "VOLKSWGN": Make.VOLKSWAGEN.value,
    "VOLKS`": Make.VOLKSWAGEN.value,
    "VOLKW": Make.VOLKSWAGEN.value,
    "VOLKWA": Make.VOLKSWAGEN.value,
    "VOLKWAGE": Make.VOLKSWAGEN.value,
    "VOLKWGN": Make.VOLKSWAGEN.value,
    "VOLLK": Make.VOLKSWAGEN.value,
    "VOLLKS": Make.VOLKSWAGEN.value,
    "VOLO": Make.VOLVO.value,
    "VOLOV": Make.VOLVO.value,
    "VOLOVO": Make.VOLVO.value,
    "VOLS": Make.VOLKSWAGEN.value,
    "VOLSWAGE": Make.VOLKSWAGEN.value,
    "VOLSWGN": Make.VOLKSWAGEN.value,
    "VOLV /": Make.VOLVO.value,
    "VOLV": Make.VOLVO.value,
    "VOLV0": Make.VOLVO.value,
    "VOLVA": Make.VOLVO.value,
    "VOLVE": Make.VOLVO.value,
    "VOLVL": Make.VOLVO.value,
    "VOLVO": Make.VOLVO.value,
    "VOLVO/": Make.VOLVO.value,
    "VOLW": Make.VOLKSWAGEN.value,
    "VOLX": Make.VOLKSWAGEN.value,
    "VOVL": Make.VOLVO.value,
    "VOVLO": Make.VOLVO.value,
    "VOVLVO": Make.VOLVO.value,
    "VOVO": Make.VOLVO.value,
    "VOYAGER": Make.PLYMOUTH.value,
    "VW": Make.VOLKSWAGEN.value,
    "WHITE GMC": Make.GMC.value,
    "WHITE VOLVO": Make.VOLVO.value,
    "WHITE": Make.WHITE.value,
    "WHITEGMC": Make.GMC.value,
    "WINN": Make.WINNEBAGO.value,
    "WINNE": Make.WINNEBAGO.value,
    "WINNEBAG": Make.WINNEBAGO.value,
    "WINNEBAGO": Make.WINNEBAGO.value,
    "WINNI": Make.WINNEBAGO.value,
    "WNBG": Make.WINNEBAGO.value,
    "WNBGO": Make.WINNEBAGO.value,
    "YAH": Make.YAMAHA.value,
    "YAHA": Make.YAMAHA.value,
    "YAHAMA": Make.YAMAHA.value,
    "YAHMA": Make.YAMAHA.value,
    "YAM": Make.YAMAHA.value,
    "YAMA": Make.YAMAHA.value,
    "YAMAH": Make.YAMAHA.value,
    "YAMAHA": Make.YAMAHA.value,
    "YAMAMA": Make.YAMAHA.value,
    "YAMH": Make.YAMAHA.value,
    "________": Make.NONE.value,
}



MOVEMENT_PRECEDING = {
    "A": "stopped",
    "B": "proceeding straight",
    "C": "ran off road",
    "D": "making right turn",
    "E": "making left turn",
    "F": "making u-turn",
    "G": "backing",
    "H": "slowing/stopping",
    "I": "passing other vehicle",
    "J": "changing lanes",
    "K": "parking maneuver",
    "L": "entering traffic",
    "M": "other unsafe turning",
    "N": "crossed into opposing lane",
    "O": "parked",
    "P": "merging",
    "Q": "traveling wrong way",
    "R": "other",
}

STATEWIDE_VEHICLE_TYPE = {
    "A": "passenger car",
    "B": "passenger car with trailer",
    "C": "motorcycle or scooter",
    "D": "pickup or panel truck",
    "E": "pickup or panel truck with trailer",
    "F": "truck or truck tractor",
    "G": "truck or truck tractor with trailer",
    "H": "schoolbus",
    "I": "other bus",
    "J": "emergency vehicle",
    "K": "highway construction equipment",
    "L": "bicycle",
    "M": "other vehicle",
    "N": "pedestrian",
    "O": "moped",
}

DEGREE_OF_INJURY = {
    "0": "no injury",
    "1": "killed",
    "2": "severe injury",
    "3": "other visible injury",
    "4": "complaint of pain",
    "5": "suspected serious injury",
    "6": "suspected minor injury",
    "7": "possible injury",
}

PARTY_TYPE = {
    "1": "driver",
    "2": "pedestrian",
    "3": "parked vehicle",
    "4": "bicyclist",
    "5": "other",
}

SEX = {
    "M": "male",
    "F": "female",
}

WEATHER = {
    "A": "clear",
    "B": "cloudy",
    "C": "raining",
    "D": "snowing",
    "E": "fog",
    "F": "other",
    "G": "wind",
}

BEAT_TYPE = {
    "0": "not chp",
    "1": "chp state highway",
    "2": "chp county roadline",
    "3": "chp county roadarea",
    "4": "schoolbus on city roadway",
    "5": "schoolbus not on public roadway",
    "6": "offroad",
    "7": "vista point, rest area, scales, or inspection facility",
    "8": "other public access",
}

CHP_BEAT_TYPE = {
    "0": "not chp",
    "1": "interstate",
    "2": "us highway",
    "3": "state route",
    "4": "county road line",
    "5": "county road area",
    "6": "us highway",
    "7": "state route",
    "8": "county road line",
    "9": "county road area",
    "10": "safety services program",
    "S": "safety services program",
    "11": "administrative",
    "A": "administrative",
}

COLLISION_TYPE = {
    "A": "head-on",
    "B": "sideswipe",
    "C": "rear end",
    "D": "broadside",
    "E": "hit object",
    "F": "overturned",
    "G": "pedestrian",
    "H": "other",
}

INVOLVED_WITH = {
    "A": "non-collision",
    "B": "pedestrian",
    "C": "other motor vehicle",
    "D": "motor vehicle on other roadway",
    "E": "parked motor vehicle",
    "F": "train",
    "G": "bicycle",
    "H": "animal",
    "I": "fixed object",
    "J": "other object",
}

PEDESTRIAN_ACTION = {
    "A": "no pedestrian involved",
    "B": "crossing in intersection crosswalk",
    "C": "crossing non-intersection crosswalk",
    "D": "crossing not in crosswalk",
    "E": "in road",
    "F": "not in road",
    "G": "using school bus",
}

DIRECTION = {
    "N": "north",
    "E": "east",
    "S": "south",
    "W": "west",
}

LOCATION_TYPE = {
    "H": "highway",
    "I": "intersection",
    "R": "ramp",
}

CHP_BEAT_CLASS = {
    "1": "chp primary",
    "2": "chp other",
    "0": "not chp",
}

HIT_AND_RUN = {
    "F": "felony",
    "M": "misdemeanor",
    "N": "not hit and run",
}

ROAD_SURFACE = {
    "A": "dry",
    "B": "wet",
    "C": "snowy",
    "D": "slippery",
}

ROAD_CONDITION = {
    "A": "holes",
    "B": "loose material",
    "C": "obstruction",
    "D": "construction",
    "E": "reduced width",
    "F": "flooded",
    "G": "other",
    "H": "normal",
}

LIGHTING = {
    "A": "daylight",
    "B": "dusk or dawn",
    "C": "dark with street lights",
    "D": "dark with no street lights",
    "E": "dark with street lights not functioning",
}

CONTROL_DEVICE = {
    "A": "functioning",
    "B": "not functioning",
    "C": "obscured",
    "D": "none",
}

SIDE_OF_HIGHWAY = {
    "N": "northbound",
    "S": "southbound",
    "E": "eastbound",
    "W": "westbound",
}

PRIMARY_COLLISION_FACTOR = {
    "A": "vehicle code violation",
    "B": "other improper driving",
    "C": "other than driver",
    "D": "unknown",
    "E": "fell asleep",
}

PCF_VIOLATION_CODE = {
    "B": "business",
    "C": "vehicle",
    "H": "city health",
    "I": "city ordinance",
    "O": "county ordinance",
    "P": "penal",
    "S": "streets",
    "W": "welfare",
}

RACE = {
    "A": "asian",
    "B": "black",
    "H": "hispanic",
    "O": "other",
    "W": "white",
}

COLLISION_SEVERITY = {
    "0": "property damage only",
    "1": "fatal",
    "2": "severe injury",
    "3": "other injury",
    "4": "pain",
}

PCF_VIOLATION_CATEGORY = {
    "00": "unknown",
    "01": "dui",
    "02": "impeding traffic",
    "03": "speeding",
    "04": "following too closely",
    "05": "wrong side of road",
    "06": "improper passing",
    "07": "unsafe lane change",
    "08": "improper turning",
    "09": "automobile right of way",
    "10": "pedestrian right of way",
    "11": "pedestrian violation",
    "12": "traffic signals and signs",
    "13": "hazardous parking",
    "14": "lights",
    "15": "brakes",
    "16": "other equipment",
    "17": "other hazardous violation",
    "18": "other than driver (or pedestrian)",
    "21": "unsafe starting or backing",
    "22": "other improper driving",
    "23": "pedestrian dui",
    "24": "fell asleep",
}

COUNTIES = {
    "01": "alameda",
    "02": "alpine",
    "03": "amador",
    "04": "butte",
    "05": "calaveras",
    "06": "colusa",
    "07": "contra costa",
    "08": "del norte",
    "09": "el dorado",
    "10": "fresno",
    "11": "glenn",
    "12": "humboldt",
    "13": "imperial",
    "14": "inyo",
    "15": "kern",
    "16": "kings",
    "17": "lake",
    "18": "lassen",
    "19": "los angeles",
    "20": "madera",
    "21": "marin",
    "22": "mariposa",
    "23": "mendocino",
    "24": "merced",
    "25": "modoc",
    "26": "mono",
    "27": "monterey",
    "28": "napa",
    "29": "nevada",
    "30": "orange",
    "31": "placer",
    "32": "plumas",
    "33": "riverside",
    "34": "sacramento",
    "35": "san benito",
    "36": "san bernardino",
    "37": "san diego",
    "38": "san francisco",
    "39": "san joaquin",
    "40": "san luis obispo",
    "41": "san mateo",
    "42": "santa barbara",
    "43": "santa clara",
    "44": "santa cruz",
    "45": "shasta",
    "46": "sierra",
    "47": "siskiyou",
    "48": "solano",
    "49": "sonoma",
    "50": "stanislaus",
    "51": "sutter",
    "52": "tehama",
    "53": "trinity",
    "54": "tulare",
    "55": "tuolumne",
    "56": "ventura",
    "57": "yolo",
    "58": "yuba",
}

CALTRANS_COUNTY = {
    "ALA":  "alameda",
    "ALP":  "alpine",
    "AMA":  "amador",
    "BUT":  "butte",
    "CAL":  "calaveras",
    "COL":  "colusa",
    "CC":   "contra costa",
    "DN":   "del norte",
    "ED":   "el dorado",
    "FRE":  "fresno",
    "GLE":  "glenn",
    "HUM":  "humboldt",
    "IMP":  "imperial",
    "INY":  "inyo",
    "KER":  "kern",
    "KIN":  "kings",
    "LAK":  "lake",
    "LAS":  "lassen",
    "LA":   "los angeles",
    "MAD":  "madera",
    "MRN":  "marin",
    "MPA":  "mariposa",
    "MEN":  "mendocino",
    "MER":  "merced",
    "MOD":  "modoc",
    "MNO":  "mono",
    "MON":  "monterey",
    "NAP":  "napa",
    "NEV":  "nevada",
    "ORA":  "orange",
    "PLA":  "placer",
    "PLU":  "plumas",
    "RIV":  "riverside",
    "SAC":  "sacramento",
    "SBT":  "san benito",
    "SBD":  "san bernardino",
    "SD":   "san diego",
    "SF":   "san francisco",
    "SJ":   "san joaquin",
    "SLO":  "san luis obispo",
    "SM":   "san mateo",
    "SB":   "santa barbara",
    "SCL":  "santa clara",
    "SCR":  "santa cruz",
    "SHA":  "shasta",
    "SIE":  "sierra",
    "SIS":  "siskiyou",
    "SOL":  "solano",
    "SON":  "sonoma",
    "STA":  "stanislaus",
    "SUT":  "sutter",
    "TEH":  "tehama",
    "TRI":  "trinity",
    "TUL":  "tulare",
    "TUO":  "tuolumne",
    "VEN":  "ventura",
    "YOL":  "yolo",
    "YUB":  "yuba",
}

CHP_VEHICLE_TYPE = {
    "01": "passenger car, station",
    "02": "motorcycle",
    "03": "motor driven",
    "04": "bicycle",
    "05": "motorized bicycle",
    "06": "all terrain vehicle",
    "07": "sport utility vehicle",
    "08": "mini-vans",
    "09": "paratransit",
    "10": "tour bus",
    "11": "other commercial",
    "12": "non-commercial bus",
    "13": "school bus public type i",
    "14": "school bus public type ii",
    "15": "school bus private type i",
    "16": "school bus private type ii",
    "17": "school bus contractual type i",
    "18": "school bus contractual type ii",
    "19": "general public paratransit",
    "20": "public transit authority",
    "21": "two axle tank truck",
    "22": "pickups & panels",
    "23": "pickup w/camper",
    "24": "three axle tank truck",
    "25": "truck tractor",
    "26": "two axle truck",
    "27": "three or more axle truck",
    "28": "semi tank trailer",
    "29": "pull tank trailer",
    "30": "two tank trailer",
    "31": "semi",
    "32": "pull",
    "33": "two trailers (includes semi & pull)",
    "34": "boat",
    "35": "utility",
    "36": "trailer coach",
    "37": "extralegal permit load",
    "38": "pole, pipe, or logging dolly",
    "39": "three trailers",
    "40": "federally legal semi",
    "41": "ambulance",
    "42": "dune buggy",
    "43": "fire truck",
    "44": "fork lift",
    "45": "hwy. construction equip.",
    "46": "implement of husbandry",
    "47": "motor home",
    "48": "police car",
    "49": "police motorcycle",
    "50": "mobile equipment",
    "51": "farm labor vehicle (certified)",
    "52": "federally legal double combo over 75 feet",
    "53": "fifth wheel travel trailer",
    "54": "container chassis",
    "55": "two-axle tow truck",
    "56": "three-axle tow truck",
    "57": "farm labor vehicle",
    "58": "farm labor transporter",
    "59": "motor home > 40 feet",
    "60": "pedestrian",
    "61": "second or additional enforcement action(s)",
    "62": "passengers",
    "63": "youth bus",
    "64": "school pupil activity bus type i",
    "65": "school pupil activity bus type ii",
    "71": "passenger car, station wagon, jeep: hazardous material",
    "72": "pickups and panels: hazardous material",
    "73": "pickup and camper: hazardous material",
    "75": "truck tractor: hazardous material",
    "76": "two-axle truck: hazardous material",
    "77": "three or more axle truck: hazardous material",
    "78": "two-axle tank truck: hazardous material",
    "79": "three-axle tank truck: hazardous material",
    "81": "passenger car, station wagon, jeep: hazardous waste or hazardous waste/material combination",
    "82": "pickups and panels: hazardous waste or hazardous waste/material combination",
    "83": "pickup and camper: hazardous waste or hazardous waste/material combination",
    "85": "truck tractor: hazardous waste or hazardous waste/material combination",
    "86": "two-axle truck: hazardous waste or hazardous waste/material combination",
    "87": "three or more axle truck: hazardous waste or hazardous waste/material combination",
    "88": "two-axle tank truck: hazardous waste or hazardous waste/material combination",
    "89": "three-axle tank truck: hazardous waste or hazardous waste/material combination",
    "94": "go-ped, zip electric scooter, motoboard",
    "95": "misc. non-motor vehicle",
    "96": "misc. motor vehicle (snowmobile, golf cart)",
    "97": "low speed vehicle",
    "98": "emergency vehicle (on emergency run)",
    "99": "unknown hit and run vehicle involvement",
}

CELLPHONE_USE_TYPE = {
    'B': "cellphone in use",
    'C': "cellphone not in use",
    'D': "no cellphone/unknown",
    '1': "cellphone in use (handheld)",
    '2': "cellphone in use (hands-free)",
    '3': "cellphone not in use",
}

SOBRIETY = {
    'A': 'had not been drinking',
    'B': 'had been drinking, under influence',
    'C': 'had been drinking, not under influence',
    'D': 'had been drinking, impairment unknown',
    'G': 'impairment unknown',
    'H': 'not applicable',
}

DRUG = {
    'E': 'under drug influence',
    'F': 'impairment - physical',
    'H': 'not applicable',
    'I': 'sleepy/fatigued',
}

SAFETY = {
    'A': 'none in vehicle',
    'B': 'unknown',
    'C': 'lap belt used',
    'D': 'lap belt not used',
    'E': 'shoulder harness used',
    'F': 'shoulder harness not used',
    'G': 'lap/shoulder harness used',
    'H': 'lap/shoulder harness not used',
    'J': 'passive restraint used',
    'K': 'passive restraint not used',
    'L': 'air bag deployed',
    'M': 'air bag not deployed',
    'N': 'other',
    'P': 'not required',
    'Q': 'child restraint in vehicle used',
    'R': 'child restraint in vehicle not used',
    'S': 'child restraint in vehicle, use unknown',
    'T': 'child restraint in vehicle, improper use',
    'U': 'no child restraint in vehicle',
    'V': 'driver, motorcycle helmet not used',
    'W': 'driver, motorcycle helmet used',
    'X': 'passenger, motorcycle helmet not used',
    'Y': 'passenger, motorcycle helmet used',
}

FINANCIAL = {
    'N': 'no proof of insurance obtained',
    'Y': 'proof of insurance obtained',
    'O': 'not applicable',
    'E': 'officer called away before obtained',
}

OAF_VIOLATION_CODE = {
    'B': 'business and professions',
    'C': 'vehicle',
    'H': 'city health and safety',
    'I': 'city ordinance',
    'O': 'county ordinance',
    'P': 'penal',
    'S': 'streets and highways',
    'W': 'welfare and institutions',
}

OAF_VIOLATION_CATEGORY = {
    '01': 'under influence in public (647f)',
    '02': 'county ordinance',
    '03': 'city ordinance',
    '05': 'business/professions code',
    '06': 'felony penal code',
    '08': 'controlled substances (felony health and safety)',
    '09': 'health/safety code (misdemeanor)',
    '10': 'penal code (misdemeanor)',
    '11': 'streets/highways code',
    '13': 'welfare/institutions code',
    '15': 'manslaughter',
    '16': 'non-vehicle code not specified above',
    '17': 'fish & game code',
    '18': 'agriculture code',
    '19': 'hit and run',
    '20': 'driving or bicycling under the influence of alcohol or drug',
    '21': 'improper lane change',
    '22': 'impeding traffic',
    '23': 'failure to heed stop signal',
    '24': 'failure to heed stop sign',
    '25': 'unsafe speed',
    '26': 'reckless driving',
    '27': 'wrong side of road',
    '28': 'unsafe lane change',
    '29': 'improper passing',
    '30': 'following too closely',
    '31': 'improper turning',
    '33': 'automobile right-of-way',
    '34': 'pedestrian right-of-way',
    '35': 'pedestrian violation',
    '38': 'hazardous parking',
    '39': 'lights',
    '40': 'brakes',
    '43': 'other equipment',
    '44': 'other hazardous movement',
    '46': 'improper registration',
    '47': 'other non-moving violation',
    '48': 'excessive smoke',
    '49': 'excessive noise',
    '50': 'overweight',
    '51': 'oversize',
    '52': 'over maximum speed',
    '53': 'unsafe starting or backing',
    '60': 'off-highway vehicle violation',
    '61': 'child restraint',
    '62': 'seat belt',
    '63': 'seat belt (equipment)',
}

OTHER_FACTOR = {
    'A': 'violation',
    'E': 'vision obscurements',
    'F': 'inattention',
    'G': 'stop and go traffic',
    'H': 'entering/leaving ramp',
    'I': 'previous collision',
    'J': 'unfamiliar with road',
    'K': 'defective vehicle equipment',
    'L': 'uninvolved vehicle',
    'M': 'other',
    'N': 'none apparent',
    'O': 'runaway vehicle',
    'P': 'inattention, cell phone',
    'Q': 'inattention, electronic equip',
    'R': 'inattention, radio/cd',
    'S': 'inattention, smoking',
    'T': 'inattention, eating',
    'U': 'inattention, children',
    'V': 'inattention, animal',
    'W': 'inattention, personal hygiene',
    'X': 'inattention, reading',
    'Y': 'inattention, other',
}

SEATING_POSITION = {
    '1': "driver",
    '2': "passenger seat 2",
    '3': "passenger seat 3",
    '4': "passenger seat 4",
    '5': "passenger seat 5",
    '6': "passenger seat 6",
    '7': "station wagon rear",
    '8': "rear occupant of truck or van",
    '9': "position unknown",
    '0': "other occupants",
}

EJECTED = {
    '0': 'not ejected',
    '1': 'fully ejected',
    '2': 'partially ejected',
    '3': 'unknown',
}

ROLE = {
    '1': 'driver',
    '2': 'passenger',
    '3': 'pedestrian',
    '4': 'bicyclist',
    '5': 'other',
    '6': 'non-injured party',
}

CHP_SHIFT = {
    '1': "0600 thru 1359",
    '2': "1400 thru 2159",
    '3': "2200 thru 0559",
    '4': "chp not stated",
    '5': "not chp",
}

POPULATION = {
    '1': '<2500',
    '2': '2500 to 10000',
    '3': '10000 to 25000',
    '4': '25000 to 50000',
    '5': '50000 to 100000',
    '6': '100000 to 250000',
    '7': '>250000',
    '9': 'unincorporated',
    '0': 'university',
}

RAMP_INTERSECTION = {
    '1': 'ramp exit, last 50 feet',
    '2': 'mid-ramp',
    '3': 'ramp entry, first 50 feet',
    '4': 'not state highway, ramp-related, within 100 feet',
    '5': 'intersection',
    '6': 'not state highway, intersection-related, within 250 feet',
    '7': 'highway',
    '8': 'not state highway',
}

RAMP_TYPE = {
    'EF': 'eastbound off-ramp',
    'EO': 'eastbound on-ramp',
    'NF': 'northbound off-ramp',
    'NO': 'northbound on-ramp',
    'SF': 'southbound off-ramp',
    'SO': 'southbound on-ramp',
    'WF': 'westbound off-ramp',
    'WO': 'westbound on-ramp',
}



from enum import Enum, unique


@unique
class DataType(Enum):
    """A class used to encode the types allowed in SQLite."""

    INTEGER = "INTEGER"
    REAL = "REAL"
    TEXT = "TEXT"
    BLOB = "BLOB"
    NULL = "NULL"


# Conversion from SQLite data types to Python data types
DATATPYE_MAP = {
    DataType.INTEGER: int,
    DataType.REAL: float,
    DataType.TEXT: str,
    DataType.BLOB: str,
    DataType.NULL: None,
}



#!/usr/bin/env python3

from switrs_to_sqlite.parsers import PartyRow


# A list of tuples, with each tuple containing a row ready to be parsed, and
# the answer that should be returned from doing so. The tuple is of the form
# (row, answer)
ROWS = (
    (
        #      Case_ID     Party_Number  Party_Type    At_Fault  Party_Sex  Party_Age  Party_Sobriety            Party_Drug_Physical      Direction_Of_Travel  Party_Safety_Equipment_1      Party_Safety_Equipment_2      Financial_Responsibility           Hazardous_Materials  Cellphone_In_Use  Cellphone_Use_Type         School_Bus_Related  OAF_Violation_Code            OAF_Violation_Category  OAF_Violation_Section  OAF_Violation_Suffix  Other_Associate_Factor_1  Other_Associate_Factor_2  Party_Number_Killed  Party_Number_Injured  Movement_Preceding_Collision  Vehicle_Year  Vehicle_Make  Statewide_Vehicle_Type  CHP_Vehicle_Type_Towing                     CHP_Vehicle_Type_Towed  Party_Race
        [      '097293',   '1',          '1',          'Y',      'M',       '20',      'A',                      'H',                     'E',                 'A',                          'F',                          'N',                               'A',                 '3',                                         'E',                'C',                          '18',                   '12345',               'A',                  'N',                      'A',                      '0',                 '0',                  'M',                          '1996',       'FORD',       'A',                    '08',                                       '00',                   'W'],
        [None, '097293',   1,            'driver',     True,     'male',    20,        'had not been drinking',  'not applicable',        'east',              'none in vehicle',            'shoulder harness not used',  'no proof of insurance obtained',  True,                False,            'cellphone not in use',    True,               'vehicle',                    'agriculture code',     '12345',               'A',                  'none apparent',          'violation',              0,                   0,                    'other unsafe turning',       1996,         'ford',       'passenger car',        'mini-vans',                                '00',                   'white'],
    ),
    (
        #      Case_ID     Party_Number  Party_Type    At_Fault  Party_Sex  Party_Age  Party_Sobriety            Party_Drug_Physical      Direction_Of_Travel  Party_Safety_Equipment_1      Party_Safety_Equipment_2      Financial_Responsibility           Hazardous_Materials  Cellphone_In_Use  Cellphone_Use_Type         School_Bus_Related  OAF_Violation_Code            OAF_Violation_Category  OAF_Violation_Section  OAF_Violation_Suffix  Other_Associate_Factor_1  Other_Associate_Factor_2  Party_Number_Killed  Party_Number_Injured  Movement_Preceding_Collision  Vehicle_Year  Vehicle_Make  Statewide_Vehicle_Type  CHP_Vehicle_Type_Towing                     CHP_Vehicle_Type_Towed  Party_Race
        [      '0018200',  '3',          '4',          'N',      'F',       '999',     'H',                      'E',                     'N',                 'G',                          '-',                          'Y',                               '' ,                 'B',                                         '-',                'W',                          '29',                   '',                    'B',                  'G',                      '-',                      '1',                 '1',                  'A',                          '1987',       'BMW',        'O',                    '27',                                       '30',                   'H'],
        [None, '0018200',  3,            'bicyclist',  False,    'female',  999,       'not applicable',         'under drug influence',  'north',             'lap/shoulder harness used',  None,                         'proof of insurance obtained',     None,                True,             'cellphone in use',        None,               'welfare and institutions',   'improper passing',     None,                  'B',                  'stop and go traffic',    None,                     1,                   1,                    'stopped',                    1987,         'bmw',        'moped',                'three or more axle truck',                 'two tank trailer',     'hispanic'],
    ),
    (
        #      Case_ID     Party_Number  Party_Type    At_Fault  Party_Sex  Party_Age  Party_Sobriety            Party_Drug_Physical      Direction_Of_Travel  Party_Safety_Equipment_1      Party_Safety_Equipment_2      Financial_Responsibility           Hazardous_Materials  Cellphone_In_Use  Cellphone_Use_Type         School_Bus_Related  OAF_Violation_Code            OAF_Violation_Category  OAF_Violation_Section  OAF_Violation_Suffix  Other_Associate_Factor_1  Other_Associate_Factor_2  Party_Number_Killed  Party_Number_Injured  Movement_Preceding_Collision  Vehicle_Year  Vehicle_Make  Statewide_Vehicle_Type  CHP_Vehicle_Type_Towing                     CHP_Vehicle_Type_Towed  Party_Race
        [      '0036598',  '2',          '-',          '-',      '-',       '998',     '-',                      '-',                     '-',                 '-',                          '-',                          '',                                '-',                 'D',                                         '',                 '-',                          '00',                   '',                    '',                   '-',                      '-',                      '20',                '2',                  '-',                          '9999',       'NOT_MAP',    '-',                    '99',                                       '',                     ''],
        [None, '0036598',  2,            None,         None,     None,      None,      None,                     None,                    None,                None,                         None,                         None,                              None,                None,             'no cellphone/unknown',    None,               None,                         None,                   None,                  None,                 None,                     None,                     20,                  2,                    None,                         None,         'NOT_MAP',    None,                   'unknown hit and run vehicle involvement',  None,                   None],
    ),
)


def test_partyrows():
    for row, answer in ROWS:
        parsed_row = PartyRow.parse_row(row)
        assert parsed_row == answer

def test_partyrow_create_table():
    assert PartyRow.create_table_statement() == (
        "CREATE TABLE "
        "parties ("
        "id INTEGER PRIMARY KEY, "
        "case_id TEXT, "
        "party_number INTEGER, "
        "party_type TEXT, "
        "at_fault INTEGER, "
        "party_sex TEXT, "
        "party_age INTEGER, "
        "party_sobriety TEXT, "
        "party_drug_physical TEXT, "
        "direction_of_travel TEXT, "
        "party_safety_equipment_1 TEXT, "
        "party_safety_equipment_2 TEXT, "
        "financial_responsibility TEXT, "
        "hazardous_materials INTEGER, "
        "cellphone_in_use INTEGER, "
        "cellphone_use_type TEXT, "
        "school_bus_related INTEGER, "
        "oaf_violation_code TEXT, "
        "oaf_violation_category TEXT, "
        "oaf_violation_section TEXT, "
        "oaf_violation_suffix TEXT, "
        "other_associate_factor_1 TEXT, "
        "other_associate_factor_2 TEXT, "
        "party_number_killed INTEGER, "
        "party_number_injured INTEGER, "
        "movement_preceding_collision TEXT, "
        "vehicle_year INTEGER, "
        "vehicle_make TEXT, "
        "statewide_vehicle_type TEXT, "
        "chp_vehicle_type_towing TEXT, "
        "chp_vehicle_type_towed TEXT, "
        "party_race TEXT"
        ")"
    )

def test_partyrow_columns():
    assert PartyRow.columns == [
        ("id", "INTEGER", "PRIMARY KEY"),
        ("case_id", "TEXT"),
        ("party_number", "INTEGER"),
        ("party_type", "TEXT"),
        ("at_fault", "INTEGER"),
        ("party_sex", "TEXT"),
        ("party_age", "INTEGER"),
        ("party_sobriety", "TEXT"),
        ("party_drug_physical", "TEXT"),
        ("direction_of_travel", "TEXT"),
        ("party_safety_equipment_1", "TEXT"),
        ("party_safety_equipment_2", "TEXT"),
        ("financial_responsibility", "TEXT"),
        ("hazardous_materials", "INTEGER"),
        ("cellphone_in_use", "INTEGER"),
        ("cellphone_use_type", "TEXT"),
        ("school_bus_related", "INTEGER"),
        ("oaf_violation_code", "TEXT"),
        ("oaf_violation_category", "TEXT"),
        ("oaf_violation_section", "TEXT"),
        ("oaf_violation_suffix", "TEXT"),
        ("other_associate_factor_1", "TEXT"),
        ("other_associate_factor_2", "TEXT"),
        ("party_number_killed", "INTEGER"),
        ("party_number_injured", "INTEGER"),
        ("movement_preceding_collision", "TEXT"),
        ("vehicle_year", "INTEGER"),
        ("vehicle_make", "TEXT"),
        ("statewide_vehicle_type", "TEXT"),
        ("chp_vehicle_type_towing", "TEXT"),
        ("chp_vehicle_type_towed", "TEXT"),
        ("party_race", "TEXT"),
    ]



#!/usr/bin/env python3

from switrs_to_sqlite.converters import convert, negative, string_to_bool, county_city_location_to_county, cellphone_use_to_bool, non_standard_str_to_bool


# Test the convert function
def test_convert():
    convert_vals = (
        # Pass through
        ("9", None, None, "9"),
        ("a", None, None, "a"),
        ("1.", None, None, "1."),
        # Standard dtypes
        ("9", int, None, 9),
        ("a", str, None, "a"),
        ("1", float, None, 1.),
        # With spaces
        ("9 ", int, None, 9),
        ("a ", str, None, "a"),
        ("1 ", float, None, 1.),
        (" 9", int, None, 9),
        (" a", str, None, "a"),
        (" 1", float, None, 1.),
        # Nulls that do nothing
        ("9", int, [""], 9),
        ("a", str, [""], "a"),
        ("1", float, [""], 1.),
        # Nulls that return None
        ("9", int, ["9"], None),
        ("a", str, ["a"], None),
        ("1.", float, ["1."], None),
        ("9", None, ["9"], None),
        ("a", None, ["a"], None),
        ("1.", None, ["1."], None),
        # Conversion failure
        ("a", int, None, None),
    )
    for val, dtype, nulls, res in convert_vals:
        assert convert(val=val, dtype=dtype, nulls=nulls) == res


# Test the negative function
def test_negative():
    convert_vals = (
        # Pass through
        ("9", None, None, ""),
        ("a", None, None, ""),
        ("1.", None, None, ""),
        # Standard dtypes
        ("9", int, None, -9),
        ("a", str, None, ""),
        ("1", float, None, -1.),
        # With spaces
        ("9 ", int, None, -9),
        ("a ", str, None, ""),
        ("1 ", float, None, -1.),
        (" 9", int, None, -9),
        (" a", str, None, ""),
        (" 1", float, None, -1.),
        # Nulls that do nothing
        ("9", int, [""], -9),
        ("a", str, [""], ""),
        ("1", float, [""], -1.),
        # Nulls that return None
        ("9", int, ["9"], None),
        ("a", str, ["a"], None),
        ("1.", float, ["1."], None),
        ("9", None, ["9"], None),
        ("a", None, ["a"], None),
        ("1.", None, ["1."], None),
        # Conversion failure
        ("a", int, None, None),
    )
    for val, dtype, nulls, res in convert_vals:
        assert negative(val=val, dtype=dtype, nulls=nulls) == res


# Test the string_to_bool function
def test_bools():
    bools = (
        ("Y", True),
        ("y", True),
        ("N", False),
        ("n", False),
    )
    for val, res in bools:
        assert string_to_bool(val=val) == res


def test_nulls():
    nones = (
        ("Y", ["Y"]),
        ("y", ["y"]),
        ("N", ["N"]),
        ("n", ["n"]),
        ("", [""]),
        ("1", ["1"]),
    )
    for val, nulls in nones:
        assert string_to_bool(val=val, nulls=nulls) is None


def test_county_city_location_to_county():
    convert_vals = (
        ("0145", str, None, "01"),
        ("5899", str, None, "58"),
        # Nulls that return None
        ("9", int, ["9"], None),
        ("a", str, ["a"], None),
        ("1.", float, ["1."], None),
        ("9", None, ["9"], None),
        ("a", None, ["a"], None),
        ("1.", None, ["1."], None),
        # Conversion failure
        ("a", int, None, None),
    )
    for val, dtype, nulls, res in convert_vals:
        assert county_city_location_to_county(val=val, dtype=dtype, nulls=nulls) == res


def test_cellphone_use_to_bool():
    convert_vals = (
        ("B", int, None, True),
        ("C", int, None, False),
        ("D", int, None, None),
        ("1", int, None, True),
        ("2", int, None, True),
        ("3", int, None, False),
        # Nulls that return None
        ("9", int, ["9"], None),
        ("a", str, ["a"], None),
        ("1.", float, ["1."], None),
        ("9", None, ["9"], None),
        ("a", None, ["a"], None),
        ("1.", None, ["1."], None),
        # Conversion failure
        ("a", int, None, None),
    )
    for val, dtype, nulls, res in convert_vals:
        assert cellphone_use_to_bool(val=val, dtype=dtype, nulls=nulls) == res


def test_non_standard_str_to_bool():
    convert_vals = (
        ("A", int, None, True),
        ("E", int, None, True),
        # Unknown keys
        ("D", int, None, None),
        ("1", int, None, None),
        ("2", int, None, None),
        ("3", int, None, None),
        # Nulls that return None
        ("9", int, ["9"], None),
        ("a", str, ["a"], None),
        ("1.", float, ["1."], None),
        ("9", None, ["9"], None),
        ("a", None, ["a"], None),
        ("1.", None, ["1."], None),
        # Conversion failure
        ("a", int, None, None),
    )
    for val, dtype, nulls, res in convert_vals:
        assert non_standard_str_to_bool(val=val, dtype=dtype, nulls=nulls) == res



from collections import OrderedDict
from datetime import datetime

from switrs_to_sqlite.datatypes import DataType, DATATPYE_MAP
from switrs_to_sqlite.row_types import (
    COLLISION_ROW,
    COLLISION_DATE_TABLE,
    PARTY_ROW,
    VICTIM_ROW,
)


class CSVParser:
    """The base class for all parsing classes.

    Derived classes should define override_parent() to change any
    values. Specifically, they are expected to set table_name and
    members. The members attribute is used to layout the table, determine how
    to insert values, and how to parse the strings in the row attribute.

    Attributes:
        NULLS (list): A list of string values that are always to be considered
            as NULL for all values in the row.
        row (list): A list of strings containing the information from the CSV
            row, as returned by csv.reader().
        table_name (str): The name of the table this parser fills. Used to
            create the table in SQLite.
        has_primary_column (bool): If True, then the first tuple in the members
            attribute is set to a "PRIMARY KEY", otherwise a self-incrementing
            index column is created.
        members (list): A list of tuples indicating how to parse each
            field in the CSV row. The form of each tuple is as follows:
                - int: The position of the field to parse.
                - str: The name to use for the field in the table.
                - DataType: The type to use to store the field value in the
                    table.
                - List OR None: List of additional values to consider NULL.
                    None if no additional values are needed.
                - function: A function to convert the value from the string it
                    is read as to its final form.

        Additional attributes are set based on the names provided in the
        members tuples. They contain the values returned by the function in the
        tuple.

    """

    def __init__(
        self, parsing_table, table_name, has_primary_column, date_parsing_table
    ):
        """Set up the class and parse the CSV row.

        This method should be called by all derived classes within their own
        __init__() functions, like so:

            super().__init__(row)

        Args:
            row (list): A list of strings containing the information from the
                CSV row, as returned by csv.reader().
        """
        self.parsing_table = parsing_table
        self.table_name = table_name
        self.has_primary_column = has_primary_column
        self.date_parsing_table = date_parsing_table

        self.__datatype_convert = DATATPYE_MAP

        # Set up column names
        self.__set_columns()

    def parse_row(self, row):
        # The CSV file is malformed, so extend it to avoid KeyErrors
        extended_row = self.__extend_row(row)

        # Set up list of variables for insertion
        values = self.__set_values(extended_row)

        return list(values.values())

    def __set_values(self, row):
        """Creates a list of the attributes set in set_variables() in the
        proper order for reading into the SQLite table.

        """
        values = OrderedDict()
        # If there is no column in the data that is a primary key, than we have
        # to add an automatic first column which needs a NULL inserted
        if not self.has_primary_column:
            values["PRIMARY_COLUMN"] = None

        # Parse each item in the row
        for i_csv, name, datatype, nulls, func, val_map in self.parsing_table:
            dtype = self.__datatype_convert[datatype]

            # Convert the CSV field to a value for SQL using the associated
            # conversion function
            val = func(val=row[i_csv], nulls=nulls, dtype=dtype)

            # If there is a val_map, then use that to convert the value to a
            # return value. This is mainly used to convert "Enums" in the
            # database into human readable values, like 'A' -> 'Stopped'.
            if val_map is not None:
                val = val_map.get(val, val)

            values[name] = val

        # Convert dates as well
        if self.date_parsing_table:
            values["collision_date"] = self.__convert_date(row, "collision_date")
            values["collision_time"] = self.__convert_time(row)
            values["process_date"] = self.__convert_date(row, "process_date")

        return values

    def __convert_date(self, row, test_name):
        # Set up the processing date
        for i, name, _ in self.date_parsing_table:
            if name == test_name:
                obj = datetime.strptime(row[i], "%Y%m%d")
                return obj.date().isoformat()

    def __convert_time(self, row):
        # Find the correct index for the
        index = None
        for i, name, _ in self.date_parsing_table:
            if name == "collision_time":
                index = i
                break

        # Set up the collision time
        # 2500 is used as NULL in the source
        collision_time_str = row[index]
        if collision_time_str == "2500":
            time = None
        else:
            # The source data is not always 0 padded, so it will be 900 instead
            # of 0900, and so length 3
            missing_leading_zero_length = 3
            if len(collision_time_str) == missing_leading_zero_length:
                collision_time_str = "0" + collision_time_str

            collision_time_obj = datetime.strptime(collision_time_str, "%H%M")
            time = collision_time_obj.time().isoformat()

        return time

    def __set_columns(self):
        """Creates a list of column names and types for the SQLite table."""
        self.columns = []
        for i_csv, name, dtype, _, _, _ in self.parsing_table:
            entry = (name, dtype.value)

            # The first item is special, it is either the "PRIMARY KEY", or we
            # need to add an ID column before it
            if i_csv == 0:
                if self.has_primary_column:
                    entry = (name, dtype.value, "PRIMARY KEY")
                else:
                    zeroth_id_column = ("id", "INTEGER", "PRIMARY KEY")
                    self.columns.append(zeroth_id_column)

            # Add the entry
            self.columns.append(entry)

        # If we have extra time columns, set those as well
        if self.date_parsing_table:
            for _, name, dtype in self.date_parsing_table:
                self.columns.append((name, dtype.value))

    def __extend_row(self, row):
        """Extend the length of the row attribute with NULL fields.

        Some rows in the CSV are incomplete and are missing columns at the end.
        This function pads these rows with NULLs so they parse correctly.

        """
        # The CSV file is malformed, not ever row is the same length, so we
        # extent it with "" which maps to null in the conversion. The +1
        # converts the final index to length.
        extend = (self.parsing_table[-1][0] + 1) - len(row)
        output_row = row + extend * [""]  # "" maps to null

        return output_row

    def insert_statement(self, values):
        """Creates an insert statement used to fill a row in the SQLite
        table."""
        vals = ["?"] * len(values)
        query = "INSERT INTO {table} VALUES ({values})".format(
            table=self.table_name, values=", ".join(vals)
        )

        return query

    def create_table_statement(self):
        """Creates a string that can be used to create the correct table in SQLite.

        Use as follows:

            with sqlite3.connect(output_file) as con:
                cursor = con.cursor()
                c = RowClass(row)
                cursor.execute(c.create_table_statement())

        """
        cols = []
        for tup in self.columns:
            cols.append(" ".join(tup))

        return "CREATE TABLE {table} ({cols})".format(
            table=self.table_name, cols=", ".join(cols)
        )


VictimRow = CSVParser(
    parsing_table=VICTIM_ROW,
    table_name="victims",
    has_primary_column=False,
    date_parsing_table=None,
)


PartyRow = CSVParser(
    parsing_table=PARTY_ROW,
    table_name="parties",
    has_primary_column=False,
    date_parsing_table=None,
)


CollisionRow = CSVParser(
    parsing_table=COLLISION_ROW,
    table_name="collisions",
    has_primary_column=True,
    date_parsing_table=COLLISION_DATE_TABLE,
)



import gzip


def open_record_file(file_name, errors=None):
    """Open a Record file, even if GZipped.

    Args:
        file_name (str): The name of a file. If the file ends in ".gz" it will
            be read a gzipped file, otherwise it will be assumed to be text.

    """
    if file_name.endswith(".gz"):
        return gzip.open(file_name, "rt", errors=errors)
    return open(file_name, "rt", errors=errors)



from switrs_to_sqlite.converters import convert, negative, string_to_bool, county_city_location_to_county, cellphone_use_to_bool, non_standard_str_to_bool
from switrs_to_sqlite.datatypes import DataType
import switrs_to_sqlite.make_map as mm
import switrs_to_sqlite.value_maps as vm


DEFAULT_NULLS = ["", "-"]

COLLISION_ROW = (
    (0, "case_id", DataType.TEXT, None, convert, None),
    (3, "jurisdiction", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (6, "officer_id", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (7, "reporting_district", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (9, "chp_shift", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_SHIFT),
    (10, "population", DataType.TEXT, DEFAULT_NULLS, convert, vm.POPULATION),
    (11, "county_city_location", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (11, "county_location", DataType.TEXT, DEFAULT_NULLS, county_city_location_to_county, vm.COUNTIES),
    (12, "special_condition", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (13, "beat_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.BEAT_TYPE),
    (14, "chp_beat_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_BEAT_TYPE),
    (15, "city_division_lapd", DataType.TEXT, DEFAULT_NULLS + ["0"], convert, None),
    (16, "chp_beat_class", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_BEAT_CLASS),
    (17, "beat_number", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (18, "primary_road", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (19, "secondary_road", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (20, "distance", DataType.REAL, DEFAULT_NULLS, convert, None),
    (21, "direction", DataType.TEXT, DEFAULT_NULLS, convert, vm.DIRECTION),
    (22, "intersection", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (23, "weather_1", DataType.TEXT, DEFAULT_NULLS + ['N', 'Y'], convert, vm.WEATHER),
    (24, "weather_2", DataType.TEXT, DEFAULT_NULLS + ['N', 'Y'], convert, vm.WEATHER),
    (25, "state_highway_indicator", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (26, "caltrans_county", DataType.TEXT, DEFAULT_NULLS, convert, vm.CALTRANS_COUNTY),
    (27, "caltrans_district", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (28, "state_route", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (29, "route_suffix", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (30, "postmile_prefix", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (31, "postmile", DataType.REAL, DEFAULT_NULLS, convert, None),
    (32, "location_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.LOCATION_TYPE),
    (33, "ramp_intersection", DataType.TEXT, DEFAULT_NULLS, convert, vm.RAMP_INTERSECTION),
    (34, "side_of_highway", DataType.TEXT, DEFAULT_NULLS, convert, vm.SIDE_OF_HIGHWAY),
    (35, "tow_away", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (36, "collision_severity", DataType.TEXT, DEFAULT_NULLS, convert, vm.COLLISION_SEVERITY),
    (37, "killed_victims", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (38, "injured_victims", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (39, "party_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (40, "primary_collision_factor", DataType.TEXT, DEFAULT_NULLS, convert, vm.PRIMARY_COLLISION_FACTOR),
    (41, "pcf_violation_code", DataType.TEXT, DEFAULT_NULLS, convert, vm.PCF_VIOLATION_CODE),
    (42, "pcf_violation_category", DataType.TEXT, DEFAULT_NULLS, convert, vm.PCF_VIOLATION_CATEGORY),
    (43, "pcf_violation", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (44, "pcf_violation_subsection", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (45, "hit_and_run", DataType.TEXT, DEFAULT_NULLS, convert, vm.HIT_AND_RUN),
    (46, "type_of_collision", DataType.TEXT, DEFAULT_NULLS + ["M"], convert, vm.COLLISION_TYPE),
    (47, "motor_vehicle_involved_with", DataType.TEXT, DEFAULT_NULLS, convert, vm.INVOLVED_WITH),
    (48, "pedestrian_action", DataType.TEXT, DEFAULT_NULLS, convert, vm.PEDESTRIAN_ACTION),
    (49, "road_surface", DataType.TEXT, DEFAULT_NULLS, convert, vm.ROAD_SURFACE),
    (50, "road_condition_1", DataType.TEXT, DEFAULT_NULLS, convert, vm.ROAD_CONDITION),
    (51, "road_condition_2", DataType.TEXT, DEFAULT_NULLS, convert, vm.ROAD_CONDITION),
    (52, "lighting", DataType.TEXT, DEFAULT_NULLS, convert, vm.LIGHTING),
    (53, "control_device", DataType.TEXT, DEFAULT_NULLS, convert, vm.CONTROL_DEVICE),
    (54, "chp_road_type", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (55, "pedestrian_collision", DataType.INTEGER, None, string_to_bool, None),
    (56, "bicycle_collision", DataType.INTEGER, None, string_to_bool, None),
    (57, "motorcycle_collision", DataType.INTEGER, None, string_to_bool, None),
    (58, "truck_collision", DataType.INTEGER, None, string_to_bool, None),
    (59, "not_private_property", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (60, "alcohol_involved", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (61, "statewide_vehicle_type_at_fault", DataType.TEXT, DEFAULT_NULLS, convert, vm.STATEWIDE_VEHICLE_TYPE),
    (62, "chp_vehicle_type_at_fault", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_VEHICLE_TYPE),
    (63, "severe_injury_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (64, "other_visible_injury_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (65, "complaint_of_pain_injury_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (66, "pedestrian_killed_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (67, "pedestrian_injured_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (68, "bicyclist_killed_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (69, "bicyclist_injured_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (70, "motorcyclist_killed_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (71, "motorcyclist_injured_count", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (72, "primary_ramp", DataType.TEXT, DEFAULT_NULLS, convert, vm.RAMP_TYPE),
    (73, "secondary_ramp", DataType.TEXT, DEFAULT_NULLS, convert, vm.RAMP_TYPE),
    (74, "latitude", DataType.REAL, DEFAULT_NULLS, convert, None),
    (75, "longitude", DataType.REAL, DEFAULT_NULLS, negative, None),
)

COLLISION_DATE_TABLE = (
    (4, "collision_date", DataType.TEXT),
    (5, "collision_time", DataType.TEXT),
    (2, "process_date", DataType.TEXT),
)

PARTY_ROW = (
    (0, "case_id", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (1, "party_number", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (2, "party_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.PARTY_TYPE),
    (3, "at_fault", DataType.INTEGER, DEFAULT_NULLS, string_to_bool, None),
    (4, "party_sex", DataType.TEXT, DEFAULT_NULLS, convert, vm.SEX),
    (5, "party_age", DataType.INTEGER, DEFAULT_NULLS + ["998"], convert, None),
    (6, "party_sobriety", DataType.TEXT, DEFAULT_NULLS, convert, vm.SOBRIETY),
    (7, "party_drug_physical", DataType.TEXT, DEFAULT_NULLS, convert, vm.DRUG),
    (8, "direction_of_travel", DataType.TEXT, DEFAULT_NULLS, convert, vm.DIRECTION),
    (9, "party_safety_equipment_1", DataType.TEXT, DEFAULT_NULLS, convert, vm.SAFETY),
    (10, "party_safety_equipment_2", DataType.TEXT, DEFAULT_NULLS, convert, vm.SAFETY),
    (11, "financial_responsibility", DataType.TEXT, DEFAULT_NULLS, convert, vm.FINANCIAL),
    (12, "hazardous_materials", DataType.INTEGER, DEFAULT_NULLS, non_standard_str_to_bool, None),
    (13, "cellphone_in_use", DataType.INTEGER, DEFAULT_NULLS, cellphone_use_to_bool, None),
    (13, "cellphone_use_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.CELLPHONE_USE_TYPE),
    (14, "school_bus_related", DataType.INTEGER, DEFAULT_NULLS, non_standard_str_to_bool, None),
    (15, "oaf_violation_code", DataType.TEXT, DEFAULT_NULLS, convert, vm.OAF_VIOLATION_CODE),
    (16, "oaf_violation_category", DataType.TEXT, DEFAULT_NULLS + ["00"], convert, vm.OAF_VIOLATION_CATEGORY),
    (17, "oaf_violation_section", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (18, "oaf_violation_suffix", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (19, "other_associate_factor_1", DataType.TEXT, DEFAULT_NULLS, convert, vm.OTHER_FACTOR),
    (20, "other_associate_factor_2", DataType.TEXT, DEFAULT_NULLS, convert, vm.OTHER_FACTOR),
    (21, "party_number_killed", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (22, "party_number_injured", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (23, "movement_preceding_collision", DataType.TEXT, DEFAULT_NULLS, convert, vm.MOVEMENT_PRECEDING),
    (24, "vehicle_year", DataType.INTEGER, DEFAULT_NULLS + ["9999"], convert, None),
    (25, "vehicle_make", DataType.TEXT, DEFAULT_NULLS, convert, mm.MAKE_MAP),
    (26, "statewide_vehicle_type", DataType.TEXT, DEFAULT_NULLS, convert, vm.STATEWIDE_VEHICLE_TYPE),
    (27, "chp_vehicle_type_towing", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_VEHICLE_TYPE),
    (28, "chp_vehicle_type_towed", DataType.TEXT, DEFAULT_NULLS, convert, vm.CHP_VEHICLE_TYPE),
    (29, "party_race", DataType.TEXT, DEFAULT_NULLS, convert, vm.RACE),
)

VICTIM_ROW = (
    (0, "case_id", DataType.TEXT, DEFAULT_NULLS, convert, None),
    (1, "party_number", DataType.INTEGER, DEFAULT_NULLS, convert, None),
    (2, "victim_role", DataType.TEXT, DEFAULT_NULLS, convert, vm.ROLE),
    (3, "victim_sex", DataType.TEXT, DEFAULT_NULLS, convert, vm.SEX),
    (4, "victim_age", DataType.INTEGER, DEFAULT_NULLS + ["998"], convert, None),
    (5, "victim_degree_of_injury", DataType.TEXT, DEFAULT_NULLS, convert, vm.DEGREE_OF_INJURY),
    (6, "victim_seating_position", DataType.TEXT, DEFAULT_NULLS, convert, vm.SEATING_POSITION),
    (7, "victim_safety_equipment_1", DataType.TEXT, DEFAULT_NULLS, convert, vm.SAFETY),
    (8, "victim_safety_equipment_2", DataType.TEXT, DEFAULT_NULLS, convert, vm.SAFETY),
    (9, "victim_ejected", DataType.TEXT, DEFAULT_NULLS, convert, vm.EJECTED),
)



#!/usr/bin/env python3

from switrs_to_sqlite.parsers import CollisionRow


# A list of tuples, with each tuple containing a row ready to be parsed, and
# the answer that should be returned from doing so. The tuple is of the form
# (row, answer)
ROWS = (
    (
        # Case_ID                Collision_Year  Process_Date  Jurisdiction  Collision_Date  Collision_Time  Officer_ID  Reporting_District  DOW    CHP_Shift          Population         County_City_Location County_Location Special_Condition  Beat_Type                CHP_Beat_Type      City_Division_LAPD  CHP_Beat_Class  Beat_Number  Primary_Road                       Secondary_Road   Distance      Direction  Intersection  Weather_1  Weather_2   State_Highway_Indicator  Caltrans_County  Caltrans_District  State_Route  Route_Suffix  Postmile_Prefix  Postmile       Location_Type    Ramp_Intersection     Side_Of_Highway  Tow_Away  Collision_Severity       Killed_Victims  Injured_Victims  Party_Count  Primary_Collision_Factor   PCF_Violation_Code  PCF_Violation_Category  PCF_Violation  PCF_Violation_Subsection  Hit_And_Run         Type_Of_Collision  Motor_Vehicle_Involved_With  Ped_Action                  Road_Surface  Road_Condition_1  Road_Condition_2  Lighting                    Control_Device  CHP_Road_Type  Pedestrian_Collision  Bicycle_Collision  Motorcycle_Collision  Truck_Collision  Not_Private_Property  Alcohol_Involved  Statewide_Vehicle_Type_At_Fault  CHP_Vehicle_Type_At_Fault  Severe_Injury_Count  Other_Visible_Injury_Count  Complaint_Of_Pain_Injury_Count  Pedestrian_Killed_Count  Pedestrian_Injured_Count  Bicyclist_Killed_Count  Bicyclist_Injured_Count  Motorcyclist_Killed_Count  Motorcyclist_Injured_Count  Primary_Ramp            Secondary_Ramp        Latitude    Longitude
        ['0100010101011401155',  '2001',         '20010416',   '0000',       '20010101',     '114',          '1155qq',   '0',                '1',   '4',               '4',               '0198',                              '-',               '0',                     'A',               '',                 '0',            '073',       'DUBLIN_BL',                       'SCARLETT_CT',   '267000000',  'W',       'N',          'A',       'D',        'N',                     'ALA',           '20',              '280',       'A',          'A',             '234000.222',  'H',             '8',                  'N',             'Y',      '0',                     '999',          '999',           '999',       'A',                       'C',                '01',                   '23152',       'A',                      'F',                'A',               'I',                         'A',                        'A',          'H',              'C',              'C',                        'D',            'A',           'N',                  'N',               'N',                  'N',             'NY',                 'N',              'B',                             '01',                      '0',                 '0',                        '0',                            '0',                     '0',                      '0',                    '0',                     '0',                       '0',                        'NF',                   'EO',                 '37.7749',  '122.4194'],
        ['0100010101011401155',                                0,                                            '1155qq',   '0',                       'chp not stated',  '25000 to 50000',  '0198',              'alameda',      None,              'not chp',               'administrative',  None,               'not chp',      '073',       'DUBLIN_BL',                       'SCARLETT_CT',   267000000.,   'west',    False,        'clear',   'snowing',  False,                   'alameda',       20,                280,         'A',          'A',             234000.222,    'highway',       'not state highway',  'northbound',    True,     'property damage only',  999,            999,             999,         'vehicle code violation',  'vehicle',          'dui',                  23152,         'A',                      'felony',           'head-on',         'fixed object',              'no pedestrian involved',   'dry',        'normal',         'obstruction',    'dark with street lights',  'none',         'A',           False,                False,             False,                False,           False,                False,            'passenger car with trailer',    'passenger car, station',  0,                   0,                          0,                              0,                       0,                        0,                      0,                       0,                         0,                          'northbound off-ramp',  'eastbound on-ramp',  37.7749,    -122.4194,    "2001-01-01", "01:14:00", "2001-04-16"],
    ),
    (
        # Case_ID                Collision_Year  Process_Date  Jurisdiction  Collision_Date  Collision_Time  Officer_ID  Reporting_District  DOW    CHP_Shift          Population         County_City_Location County_Location Special_Condition  Beat_Type                CHP_Beat_Type      City_Division_LAPD  CHP_Beat_Class  Beat_Number  Primary_Road                       Secondary_Road   Distance      Direction  Intersection  Weather_1  Weather_2   State_Highway_Indicator  Caltrans_County  Caltrans_District  State_Route  Route_Suffix  Postmile_Prefix  Postmile       Location_Type    Ramp_Intersection     Side_Of_Highway  Tow_Away  Collision_Severity       Killed_Victims  Injured_Victims  Party_Count  Primary_Collision_Factor   PCF_Violation_Code  PCF_Violation_Category  PCF_Violation  PCF_Violation_Subsection  Hit_And_Run         Type_Of_Collision  Motor_Vehicle_Involved_With  Ped_Action                  Road_Surface  Road_Condition_1  Road_Condition_2  Lighting                    Control_Device  CHP_Road_Type  Pedestrian_Collision  Bicycle_Collision  Motorcycle_Collision  Truck_Collision  Not_Private_Property  Alcohol_Involved  Statewide_Vehicle_Type_At_Fault  CHP_Vehicle_Type_At_Fault  Severe_Injury_Count  Other_Visible_Injury_Count  Complaint_Of_Pain_Injury_Count  Pedestrian_Killed_Count  Pedestrian_Injured_Count  Bicyclist_Killed_Count  Bicyclist_Injured_Count  Motorcyclist_Killed_Count  Motorcyclist_Injured_Count  Primary_Ramp            Secondary_Ramp        Latitude    Longitude
        ['3337743',              '2007',         '20080214',   '1900',       '20070821',     '0910',         '478242',   '0453',             '2',   '5',               '0',               '1949',                              '0',               '8',                     '0',               '-',                '-',            '43T1',      'HOXIE_AV',                        'IMPERIAL_HWY',  '400.02',     'S',       'Y',          '',        '-',        'Y',                     'LA',            '99',              '80',        'B',          'B',             '38',          'I',             '2',                  'E',             'N',      '4',                     '0',            '0',             '1',         'E',                       'W',                '08',                   '22107',       '',                       'N',                'E',               'J',                         'E',                        'C',          'A',              '-',              'A',                        'A',            '4',           'Y',                  'Y',               'Y',                  'Y',             'Y',                  'Y',              'N',                             '60',                      '999',               '999',                      '999',                          '999',                   '999',                    '999',                  '999',                   '999',                     '999',                      'TR',                   '-',                  '34.0522',  '118.2437'],
        ['3337743',                                            1900,                                         '478242',   '0453',                    'not chp',         'university',      '1949',              'los angeles',  '0',               'other public access',   'not chp',         None,               None,           '43T1',      'HOXIE_AV',                        'IMPERIAL_HWY',  400.02,       'south',   True,         None,      None,       True,                    'los angeles',    99,                80,          'B',          'B',             38,            'intersection',  'mid-ramp',           'eastbound',     False,    'pain',                  0,              0,               1,           'fell asleep',             'welfare',          'improper turning',     22107,         None,                     'not hit and run',  'hit object',      'other object',              'in road',                  'snowy',      'holes',          None,             'daylight',                 'functioning',  '4',           True,                 True,              True,                 True,            True,                 True,             'pedestrian',                    'pedestrian',              999,                 999,                        999,                            999,                     999,                      999,                    999,                     999,                       999,                        'TR',                   None,                 34.0522,    -118.2437,    "2007-08-21", "09:10:00", "2008-02-14"],
    ),
    (
        # Case_ID                Collision_Year  Process_Date  Jurisdiction  Collision_Date  Collision_Time  Officer_ID  Reporting_District  DOW    CHP_Shift          Population         County_City_Location County_Location Special_Condition  Beat_Type                CHP_Beat_Type      City_Division_LAPD  CHP_Beat_Class  Beat_Number  Primary_Road                       Secondary_Road   Distance      Direction  Intersection  Weather_1  Weather_2   State_Highway_Indicator  Caltrans_County  Caltrans_District  State_Route  Route_Suffix  Postmile_Prefix  Postmile       Location_Type    Ramp_Intersection     Side_Of_Highway  Tow_Away  Collision_Severity       Killed_Victims  Injured_Victims  Party_Count  Primary_Collision_Factor   PCF_Violation_Code  PCF_Violation_Category  PCF_Violation  PCF_Violation_Subsection  Hit_And_Run         Type_Of_Collision  Motor_Vehicle_Involved_With  Ped_Action                  Road_Surface  Road_Condition_1  Road_Condition_2  Lighting                    Control_Device  CHP_Road_Type  Pedestrian_Collision  Bicycle_Collision  Motorcycle_Collision  Truck_Collision  Not_Private_Property  Alcohol_Involved  Statewide_Vehicle_Type_At_Fault  CHP_Vehicle_Type_At_Fault  Severe_Injury_Count  Other_Visible_Injury_Count  Complaint_Of_Pain_Injury_Count  Pedestrian_Killed_Count  Pedestrian_Injured_Count  Bicyclist_Killed_Count  Bicyclist_Injured_Count  Motorcyclist_Killed_Count  Motorcyclist_Injured_Count  Primary_Ramp            Secondary_Ramp        Latitude    Longitude
        ['90180431',             '2015',         '20160516',   '9535',       '20151021',     '1635',         '021169',   '',                 '3',   '2',               '-',               '',                                  '3',               '-',                     '-',               'A',                '2',            '-',         'I-710_(LONG_BEACH_FREEWAY)_N/B',  'FLORAL_DR',     '-',          '-',       '-',          'N',       'Y',        '-',                     'not_a_county',  '',                '',          '',           '',              '',            '',              '',                   '',              '',       '1',                     '-',            '-',             '',          '-',                       '-',                '-',                    '-',           '-',                      'M',                '-',               '-',                         '-',                        '-',          '-',              'E',              '-',                        '-',            '',            '',                   '',                '',                   '',              '',                   '',               '-',                             '-',                       '-',                 '-',                        '-',                            '-',                     '-',                      '-',                    '-',                     '-',                       '-',                        '-',                    '-',                  '',         ''],
        ['90180431',                                           9535,                                         '021169',   None,                      '1400 thru 2159',  None,              None,                None,           '3',               None,                    None,              'A',                'chp other',    None,        'I-710_(LONG_BEACH_FREEWAY)_N/B',  'FLORAL_DR',     None,         None,      None,         None,      None,       None,                    'not_a_county',   None,              None,        None,         None,            None,          None,            None,                 None,            None,     'fatal',                 None,           None,            None,        None,                      None,               None,                   None,          None,                     'misdemeanor',      None,              None,                        None,                       None,         None,             'reduced width',  None,                       None,           None,          False,                False,             False,                False,            None,                 None,             None,                            None,                      None,                None,                       None,                           None,                    None,                     None,                   None,                    None,                      None,                       None,                  None,                 None,       None,        "2015-10-21", "16:35:00", "2016-05-16"],
    ),
)


def test_collisionrows():
    for row, answer in ROWS:
        parsed_row = CollisionRow.parse_row(row)
        assert parsed_row == answer

def test_collisionrow_create_table():
    assert CollisionRow.create_table_statement() == (
        "CREATE TABLE "
        "collisions ("
        "case_id TEXT PRIMARY KEY, "
        "jurisdiction INTEGER, "
        "officer_id TEXT, "
        "reporting_district TEXT, "
        "chp_shift TEXT, "
        "population TEXT, "
        "county_city_location TEXT, "
        "county_location TEXT, "
        "special_condition TEXT, "
        "beat_type TEXT, "
        "chp_beat_type TEXT, "
        "city_division_lapd TEXT, "
        "chp_beat_class TEXT, "
        "beat_number TEXT, "
        "primary_road TEXT, "
        "secondary_road TEXT, "
        "distance REAL, "
        "direction TEXT, "
        "intersection INTEGER, "
        "weather_1 TEXT, "
        "weather_2 TEXT, "
        "state_highway_indicator INTEGER, "
        "caltrans_county TEXT, "
        "caltrans_district INTEGER, "
        "state_route INTEGER, "
        "route_suffix TEXT, "
        "postmile_prefix TEXT, "
        "postmile REAL, "
        "location_type TEXT, "
        "ramp_intersection TEXT, "
        "side_of_highway TEXT, "
        "tow_away INTEGER, "
        "collision_severity TEXT, "
        "killed_victims INTEGER, "
        "injured_victims INTEGER, "
        "party_count INTEGER, "
        "primary_collision_factor TEXT, "
        "pcf_violation_code TEXT, "
        "pcf_violation_category TEXT, "
        "pcf_violation INTEGER, "
        "pcf_violation_subsection TEXT, "
        "hit_and_run TEXT, "
        "type_of_collision TEXT, "
        "motor_vehicle_involved_with TEXT, "
        "pedestrian_action TEXT, "
        "road_surface TEXT, "
        "road_condition_1 TEXT, "
        "road_condition_2 TEXT, "
        "lighting TEXT, "
        "control_device TEXT, "
        "chp_road_type TEXT, "
        "pedestrian_collision INTEGER, "
        "bicycle_collision INTEGER, "
        "motorcycle_collision INTEGER, "
        "truck_collision INTEGER, "
        "not_private_property INTEGER, "
        "alcohol_involved INTEGER, "
        "statewide_vehicle_type_at_fault TEXT, "
        "chp_vehicle_type_at_fault TEXT, "
        "severe_injury_count INTEGER, "
        "other_visible_injury_count INTEGER, "
        "complaint_of_pain_injury_count INTEGER, "
        "pedestrian_killed_count INTEGER, "
        "pedestrian_injured_count INTEGER, "
        "bicyclist_killed_count INTEGER, "
        "bicyclist_injured_count INTEGER, "
        "motorcyclist_killed_count INTEGER, "
        "motorcyclist_injured_count INTEGER, "
        "primary_ramp TEXT, "
        "secondary_ramp TEXT, "
        "latitude REAL, "
        "longitude REAL, "
        "collision_date TEXT, "
        "collision_time TEXT, "
        "process_date TEXT"
        ")"
    )

def test_partyrow_columns():
    assert CollisionRow.columns == [
        ("case_id", "TEXT", "PRIMARY KEY"),
        ("jurisdiction", "INTEGER"),
        ("officer_id", "TEXT"),
        ("reporting_district", "TEXT"),
        ("chp_shift", "TEXT"),
        ("population", "TEXT"),
        ("county_city_location", "TEXT"),
        ("county_location", "TEXT"),
        ("special_condition", "TEXT"),
        ("beat_type", "TEXT"),
        ("chp_beat_type", "TEXT"),
        ("city_division_lapd", "TEXT"),
        ("chp_beat_class", "TEXT"),
        ("beat_number", "TEXT"),
        ("primary_road", "TEXT"),
        ("secondary_road", "TEXT"),
        ("distance", "REAL"),
        ("direction", "TEXT"),
        ("intersection", "INTEGER"),
        ("weather_1", "TEXT"),
        ("weather_2", "TEXT"),
        ("state_highway_indicator", "INTEGER"),
        ("caltrans_county", "TEXT"),
        ("caltrans_district", "INTEGER"),
        ("state_route", "INTEGER"),
        ("route_suffix", "TEXT"),
        ("postmile_prefix", "TEXT"),
        ("postmile", "REAL"),
        ("location_type", "TEXT"),
        ("ramp_intersection", "TEXT"),
        ("side_of_highway", "TEXT"),
        ("tow_away", "INTEGER"),
        ("collision_severity", "TEXT"),
        ("killed_victims", "INTEGER"),
        ("injured_victims", "INTEGER"),
        ("party_count", "INTEGER"),
        ("primary_collision_factor", "TEXT"),
        ("pcf_violation_code", "TEXT"),
        ("pcf_violation_category", "TEXT"),
        ("pcf_violation", "INTEGER"),
        ("pcf_violation_subsection", "TEXT"),
        ("hit_and_run", "TEXT"),
        ("type_of_collision", "TEXT"),
        ("motor_vehicle_involved_with", "TEXT"),
        ("pedestrian_action", "TEXT"),
        ("road_surface", "TEXT"),
        ("road_condition_1", "TEXT"),
        ("road_condition_2", "TEXT"),
        ("lighting", "TEXT"),
        ("control_device", "TEXT"),
        ("chp_road_type", "TEXT"),
        ("pedestrian_collision", "INTEGER"),
        ("bicycle_collision", "INTEGER"),
        ("motorcycle_collision", "INTEGER"),
        ("truck_collision", "INTEGER"),
        ("not_private_property", "INTEGER"),
        ("alcohol_involved", "INTEGER"),
        ("statewide_vehicle_type_at_fault", "TEXT"),
        ("chp_vehicle_type_at_fault", "TEXT"),
        ("severe_injury_count", "INTEGER"),
        ("other_visible_injury_count", "INTEGER"),
        ("complaint_of_pain_injury_count", "INTEGER"),
        ("pedestrian_killed_count", "INTEGER"),
        ("pedestrian_injured_count", "INTEGER"),
        ("bicyclist_killed_count", "INTEGER"),
        ("bicyclist_injured_count", "INTEGER"),
        ("motorcyclist_killed_count", "INTEGER"),
        ("motorcyclist_injured_count", "INTEGER"),
        ("primary_ramp", "TEXT"),
        ("secondary_ramp", "TEXT"),
        ("latitude", "REAL"),
        ("longitude", "REAL"),
        ("collision_date", "TEXT"),
        ("collision_time", "TEXT"),
        ("process_date", "TEXT"),
    ]



#!/usr/bin/env python3

import argparse
import csv
import sqlite3

from switrs_to_sqlite.open_record import open_record_file
from switrs_to_sqlite.parsers import CollisionRow, PartyRow, VictimRow


# Library version
__version__ = "4.1.1"


def main():
    """Runs the conversion."""

    # We only need to parse command line flags if running as the main script
    argparser = argparse.ArgumentParser(
        description="Convert SWITRS text files to a SQLite3 database"
    )
    # The list of input files
    argparser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=__version__),
    )
    argparser.add_argument(
        "collision_record",
        type=str,
        help="the CollisionRecords.txt file or the same file gzipped",
    )
    argparser.add_argument(
        "party_record",
        type=str,
        help="the PartyRecords.txt file or the same file gzipped",
    )
    argparser.add_argument(
        "victim_record",
        type=str,
        help="the VictimRecords.txt file or the same file gzipped",
    )
    argparser.add_argument(
        "-p",
        "--parse-error",
        help="how to handle parsing errors",
        choices=["strict", "ignore", "replace"],
    )
    argparser.add_argument(
        "-o",
        "--output-file",
        help="file to save the database to",
        default="switrs.sqlite3",
    )

    args = argparser.parse_args()

    # Match the parsers with the files they read
    pairs = (
        (CollisionRow, args.collision_record),
        (PartyRow, args.party_record),
        (VictimRow, args.victim_record),
    )

    with sqlite3.connect(args.output_file) as con:
        for RowClass, file_name in pairs:
            # Add the table to the database
            con.execute(RowClass.create_table_statement())

            # Read in the CSV and process each row
            with open_record_file(file_name, errors=args.parse_error) as f:
                reader = csv.reader(f)
                next(reader)  # Skip the header

                # Parse each row and insert it into the database
                for row in reader:
                    parsed_row = RowClass.parse_row(row)
                    con.execute(RowClass.insert_statement(parsed_row), parsed_row)


if __name__ == "__main__":
    main()



def convert(**kwargs):
    """Convert a value to a dtype, but turn certain values to None.

    Convert calls val.strip() before performing any other work.

    Args:
        **kwargs: Two specific keywords must be passed, a third is optional:
            - val (str): A value to convert to dtype.
            - dtype (callable): A callable object that returns the desired
                type, if None then the val is passed through unchanged.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns dtype(val) if val is not in nulls, otherwise
            None. If dtype(val) raises a ValueError, None is returned.
    """
    # Get the arguments
    val = kwargs.get("val")
    dtype = kwargs.get("dtype")
    nulls = kwargs.get("nulls", None)

    # Strip spaces
    sval = val.strip()

    # Return None if the val matches a string in nulls
    if nulls is not None:
        if sval in nulls:
            return None

    # Otherwise return the converted value
    if dtype is not None:
        try:
            return dtype(sval)
        except ValueError:
            return None
    # Note: val and not sval because this is the identity operation
    else:
        return val


def negative(**kwargs):
    """Use convert() to convert a value, and then return it multiplied by -1.

    This function uses convert() to perform the conversion and so passes
    through all arguments. If convert() returns None, than this function will
    also.

    Args:
        **kwargs: Two specific keywords must be passed, a third is optional:
            - val (str): A value to convert to dtype.
            - dtype (callable): A callable object that returns the desired
                type, if None then the val is passed through unchanged. The
                returned object should allow multiplication by -1, or None will
                be returned instead.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns -1 * convert(kwargs), unless convert() returns
            None, in which case None is returned.
    """
    # Use convert to do the conversion
    out_val = convert(**kwargs)

    # If convert succeeded, return the value times -1
    if out_val is not None:
        return -1 * out_val

    return None


def string_to_bool(**kwargs):
    """Convert Y/N or y/n to a True/False, or None if in a list of nulls.

    Args:
        **kwargs: One specific keywords must be passed, a second is optional:
            - val (str): A value to convert to a bool.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns a bool if val is not in nulls, otherwise None.

    """
    # Get the arguments
    val = kwargs.get("val")
    nulls = kwargs.get("nulls", None)

    # Return None if the val matches a string in nulls
    if nulls is not None:
        if val in nulls:
            return None

    # Check if val is True, otherwise return False
    if val.lower() == "y":
        return True
    return False


def county_city_location_to_county(**kwargs):
    """Convert a 4-digit county-city location code to a county code.

    The county-city codes are four digits, like XXYY. The county code is just
    the first two digits, so XX in this example. They codes are not numbers,
    leading zeros need to be preserved.

    Args:
        **kwargs: One specific keywords must be passed, a second is optional:
            - val (str): A value to convert to a a county code.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns a bool if val is not in nulls, otherwise None.

    """
    # Use convert to do the conversion
    out_val = convert(**kwargs)

    if out_val is None:
        return None

    return out_val[:2]


def cellphone_use_to_bool(**kwargs):
    """A cellphone use code to True/False if a cellphone was in use.

    The mapping is:

    B -> cellphone in use              -> True
    C -> cellphone not in use          -> False
    D -> no cellphone/unknown          -> None
    1 -> cellphone in use (handheld)   -> True
    2 -> cellphone in use (hands-free) -> True
    3 -> cellphone not in use          -> False

    Args:
        **kwargs: One specific keywords must be passed, a second is optional:
            - val (str): A value to convert to a a county code.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns a bool if val is not in nulls, otherwise None.

    """
    # Get the arguments
    val = kwargs.get("val")
    nulls = kwargs.get("nulls", None)

    # Return None if the val matches a string in nulls
    if nulls is not None:
        if val in nulls:
            return None

    # Map val
    CELLPHONE_IN_USE = {
        'B': True,
        'C': False,
        'D': None,
        '1': True,
        '2': True,
        '3': False,
    }

    return CELLPHONE_IN_USE.get(val, None)


def non_standard_str_to_bool(**kwargs):
    """Convert a hard-code set of keys to bools, everything else to None.

    Args:
        **kwargs: One specific keywords must be passed, a second is optional:
            - val (str): A value to convert to a a county code.
            - nulls (iterable, optional): An iterable containing strings to
                check against. If val if found to be equal to a string in this
                list, None is returned.

    Returns:
        converted_val: Returns a bool if val is not in nulls, otherwise None.

    """
    # Get the arguments
    val = kwargs.get("val")
    nulls = kwargs.get("nulls", None)

    # Return None if the val matches a string in nulls
    if nulls is not None:
        if val in nulls:
            return None

    # Map val
    MAP = {
        # Parties: hazardous_materials
        'A': True,
        # Parties: school_bus_related
        'E': True,
    }

    return MAP.get(val, None)



#!/usr/bin/env python3

import re
from setuptools import setup


# Get the version from the main script
version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open("switrs_to_sqlite/main.py").read(),
    re.M,
).group(1)


# Try to import pypandoc to convert the readme, otherwise ignore it
try:
    import pypandoc

    long_description = pypandoc.convert_file("README.md", "rst")
except ImportError:
    long_description = ""


# Configure the package
setup(
    name="switrs-to-sqlite",
    version=version,
    description="Script for converting SWITRS reports to SQLite.",
    long_description=long_description,
    author="Alexander Gude",
    author_email="alex.public.account@gmail.com",
    url="https://github.com/agude/SWITRS-to-SQLite",
    license="GPLv3+",
    platforms=["any"],
    packages=["switrs_to_sqlite"],
    entry_points={
        "console_scripts": [
            "switrs_to_sqlite = switrs_to_sqlite.main:main"
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Utilities",
    ],
    keywords=[
        "switrs",
        "sqlite",
        "data",
    ],
    setup_requires=[
        "pypandoc",
        "pytest-runner",
    ],
    tests_require=["pytest"],
    python_requires=">=3.8, <4",
)



#!/usr/bin/env python3

from switrs_to_sqlite.parsers import VictimRow


# A list of tuples, with each tuple containing a row ready to be parsed, and
# the answer that should be returned from doing so. The tuple is of the form
# (row, answer)
ROWS = (
    (
        #      Case_ID     Party_Number  Victim_Role   Victim_Sex  Victim_Age  Victim_Degree_Of_Injury  Victim_Seating_Position  Victim_Safety_Equipment_1     Victim_Safety_Equipment_2                    Victim_Ejected
        [      '097293',   '1',          '5',          'M',        '20',       '0',                     '3',                     'G',                          'T',                                         '0'],
        [None, '097293',   1,            'other',      'male',     20,         'no injury',             'passenger seat 3',      'lap/shoulder harness used',  'child restraint in vehicle, improper use',  'not ejected'],
    ),
    (
        #      Case_ID     Party_Number  Victim_Role   Victim_Sex  Victim_Age  Victim_Degree_Of_Injury  Victim_Seating_Position  Victim_Safety_Equipment_1     Victim_Safety_Equipment_2                    Victim_Ejected
        [      '965874',   '2',          '2',          '-',        '998',      '-',                     'A',                     '-',                          '-',                                         '2'],
        [None, '965874',   2,            'passenger',  None,       None,       None,                    'A',                     None,                         None,                                        'partially ejected'],
    ),
    (
        #      Case_ID     Party_Number  Victim_Role   Victim_Sex  Victim_Age  Victim_Degree_Of_Injury  Victim_Seating_Position  Victim_Safety_Equipment_1     Victim_Safety_Equipment_2                    Victim_Ejected
        [      '0000003',  '6',          '1',          'F',        '999',      '7',                     '1',                     'C',                          '-',                                         '-'],
        [None, '0000003',  6,            'driver',     'female',   999,        'possible injury',       'driver',                'lap belt used',              None,                                        None],
    ),
)


def test_victimrows():
    for row, answer in ROWS:
        parsed_row = VictimRow.parse_row(row)
        assert parsed_row == answer

def test_vicitimrow_create_table():
    assert VictimRow.create_table_statement() == (
        "CREATE TABLE "
        "victims ("
        "id INTEGER PRIMARY KEY, "
        "case_id TEXT, "
        "party_number INTEGER, "
        "victim_role TEXT, "
        "victim_sex TEXT, "
        "victim_age INTEGER, "
        "victim_degree_of_injury TEXT, "
        "victim_seating_position TEXT, "
        "victim_safety_equipment_1 TEXT, "
        "victim_safety_equipment_2 TEXT, "
        "victim_ejected TEXT"
        ")"
    )

def test_victimrow_columns():
    assert VictimRow.columns == [
        ("id", "INTEGER", "PRIMARY KEY"),
        ("case_id", "TEXT"),
        ("party_number", "INTEGER"),
        ("victim_role", "TEXT"),
        ("victim_sex", "TEXT"),
        ("victim_age", "INTEGER"),
        ("victim_degree_of_injury", "TEXT"),
        ("victim_seating_position", "TEXT"),
        ("victim_safety_equipment_1", "TEXT"),
        ("victim_safety_equipment_2", "TEXT"),
        ("victim_ejected", "TEXT"),
    ]



#!/usr/bin/env python3

from switrs_to_sqlite.converters import convert, string_to_bool
from switrs_to_sqlite.datatypes import DataType
from switrs_to_sqlite.parsers import CSVParser

import pytest


@pytest.fixture(scope="module")
def row():
    return ["9", "a", "1.", "Y"]

@pytest.fixture(scope="module")
def parsing_table():
    return (
        (0, "first", DataType.INTEGER, None, convert, None),
        (1, "second", DataType.TEXT, None, convert, None),
        (2, "third", DataType.REAL, None, convert, None),
        (3, "forth", DataType.INTEGER, None, string_to_bool, None),
        (4, "blank", DataType.INTEGER, (""), convert, None),
    )

@pytest.fixture(scope="function")
def parser(parsing_table):
    Parser = CSVParser(
        parsing_table=parsing_table,
        table_name="Test",
        has_primary_column=False,
        date_parsing_table=None,
    )

    return Parser


def test_extend_row_without_has_primary_column(parser):
    parser.has_primary_column = False
    values = parser.parse_row([""])
    assert len(values) == 6


def test_extend_row_with_has_primary_column(parser):
    parser.has_primary_column = True
    values = parser.parse_row([""])
    assert len(values) == 5


def test_parse_row_without_has_primary_column(parser, row):
    parser.has_primary_column = False
    values = parser.parse_row(row)
    assert values == [None, 9, "a", 1., True, None]


def test_parse_row_with_has_primary_column(parser, row):
    parser.has_primary_column = True
    values = parser.parse_row(row)
    assert values == [9, "a", 1., True, None]


def test_set_columns_without_has_primary_column(parser):
    assert parser.columns[0] == ("id", "INTEGER", "PRIMARY KEY")
    assert parser.columns[1] == ("first", "INTEGER")
    assert parser.columns[2] == ("second", "TEXT")
    assert parser.columns[3] == ("third", "REAL")
    assert parser.columns[4] == ("forth", "INTEGER")
    assert parser.columns[5] == ("blank", "INTEGER")


def test_set_columns_with_has_primary_column(parsing_table):
    parser = CSVParser(
        parsing_table=parsing_table,
        table_name="Test",
        has_primary_column=True,
        date_parsing_table=None,
    )
    assert parser.columns[0] == ("first", "INTEGER", "PRIMARY KEY")
    assert parser.columns[1] == ("second", "TEXT")
    assert parser.columns[2] == ("third", "REAL")
    assert parser.columns[3] == ("forth", "INTEGER")
    assert parser.columns[4] == ("blank", "INTEGER")


def test_insert_statement(parser, row):
    statement = parser.insert_statement(['?', '?', '?', '?', '?'])
    assert statement == "INSERT INTO Test VALUES (?, ?, ?, ?, ?)"


def test_create_table_statement_without_has_primary_column(parser):
    parser.has_primary_column = False
    statement = parser.create_table_statement()
    assert statement == "CREATE TABLE Test (id INTEGER PRIMARY KEY, first INTEGER, second TEXT, third REAL, forth INTEGER, blank INTEGER)"


def test_create_table_statement_with_has_primary_column(parsing_table):
    parser = CSVParser(
        parsing_table=parsing_table,
        table_name="Test",
        has_primary_column=True,
        date_parsing_table=None,
    )
    statement = parser.create_table_statement()
    assert statement == "CREATE TABLE Test (first INTEGER PRIMARY KEY, second TEXT, third REAL, forth INTEGER, blank INTEGER)"



#!/usr/bin/env python3

from switrs_to_sqlite.open_record import open_record_file
import gzip
import os.path


def test_read_gzipped_file(tmpdir):
    # Write a file to read back
    f = tmpdir.join("test.csv.gz")
    contents = "Test contents\nsecond line"
    file_path = os.path.join(f.dirname, f.basename)
    with gzip.open(file_path, 'wt') as f:
        f.write(contents)

    # Read back the file
    with open_record_file(file_path) as f:
        assert f.read() == contents


def test_read_normal_file(tmpdir):
    # Write a file to read back
    f = tmpdir.join("test.csv")
    contents = "Test contents\nsecond line"
    f.write(contents)
    file_path = os.path.join(f.dirname, f.basename)

    # Read back the file
    with open_record_file(file_path) as f:
        assert f.read() == contents
